@@ -1,28 +1,30 @@
 /* Evaluate expressions for GDB.
-   Copyright (C) 1986, 1987 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1987, 1989 Free Software Foundation, Inc.
 
-GDB is distributed in the hope that it will be useful, but WITHOUT ANY
-WARRANTY.  No author or distributor accepts responsibility to anyone
-for the consequences of using it or for whether it serves any
-particular purpose or works at all, unless he says so in writing.
-Refer to the GDB General Public License for full details.
+This file is part of GDB.
 
-Everyone is granted permission to copy, modify and redistribute GDB,
-but only under the conditions described in the GDB General Public
-License.  A copy of this license is supposed to have been given to you
-along with GDB so you can know your rights and responsibilities.  It
-should be in a file named COPYING.  Among other things, the copyright
-notice and this notice must be preserved on all copies.
+GDB is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-In other words, go ahead and share GDB, but don't try to stop
-anyone else from sharing it farther.  Help stamp out software hoarding!
-*/
+GDB is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GDB; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "defs.h"
 #include "param.h"
 #include "symtab.h"
 #include "value.h"
 #include "expression.h"
+#include "target.h"
+
+#define	NULL_TYPE	((struct type *)0)
 
 
 /* Parse the string EXP as a C expression, evaluate it,
@@ -54,7 +56,7 @@ parse_and_eval_address_1 (expptr)
   register struct cleanup *old_chain
     = make_cleanup (free_current_contents, &expr);
 
-  addr = value_as_long (evaluate_expression (expr));
+  addr = (CORE_ADDR) value_as_long (evaluate_expression (expr));
   do_cleanups (old_chain);
   return addr;
 }
@@ -106,8 +108,14 @@ enum noside
 { EVAL_NORMAL,
   EVAL_SKIP,			/* Only effect is to increment pos.  */
   EVAL_AVOID_SIDE_EFFECTS,	/* Don't modify any variables or
-				   call any functions.  Correct type
-				   is returned.  */
+				   call any functions.  The value
+				   returned will have the correct
+				   type, and will have an
+				   approximately correct lvalue
+				   type (inaccuracy: anything that is
+				   listed as being in a register in
+				   the function in which it was
+				   declared will be lval_register).  */
 };
 
 value
@@ -115,7 +123,7 @@ evaluate_expression (exp)
      struct expression *exp;
 {
   int pc = 0;
-  return evaluate_subexp (0, exp, &pc, EVAL_NORMAL);
+  return evaluate_subexp (NULL_TYPE, exp, &pc, EVAL_NORMAL);
 }
 
 /* Evaluate an expression, avoiding all memory references
@@ -126,7 +134,7 @@ evaluate_type (exp)
      struct expression *exp;
 {
   int pc = 0;
-  return evaluate_subexp (0, exp, &pc, EVAL_AVOID_SIDE_EFFECTS);
+  return evaluate_subexp (NULL_TYPE, exp, &pc, EVAL_AVOID_SIDE_EFFECTS);
 }
 
 static value
@@ -150,7 +158,8 @@ evaluate_subexp (expect_type, exp, pos, noside)
     {
     case OP_SCOPE:
       tem = strlen (&exp->elts[pc + 2].string);
-      (*pos) += 3 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+      (*pos) += 3 + ((tem + sizeof (union exp_element))
+		     / sizeof (union exp_element));
       return value_static_field (exp->elts[pc + 1].type,
 				 &exp->elts[pc + 2].string, -1);
 
@@ -168,7 +177,33 @@ evaluate_subexp (expect_type, exp, pos, noside)
       (*pos) += 2;
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      return value_of_variable (exp->elts[pc + 1].symbol);
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	{
+	  struct symbol * sym = exp->elts[pc + 1].symbol;
+	  enum lval_type lv;
+
+	  switch (SYMBOL_CLASS (sym))
+	    {
+	    case LOC_CONST:
+	    case LOC_LABEL:
+	    case LOC_CONST_BYTES:
+	      lv = not_lval;
+	      break;
+
+	    case LOC_REGISTER:
+	    case LOC_REGPARM:
+	      lv = lval_register;
+	      break;
+
+	    default:
+	      lv = lval_memory;
+	      break;
+	    }
+
+	  return value_zero (SYMBOL_TYPE (sym), lv);
+	}
+      else
+	return value_of_variable (exp->elts[pc + 1].symbol);
 
     case OP_LAST:
       (*pos) += 2;
@@ -184,23 +219,24 @@ evaluate_subexp (expect_type, exp, pos, noside)
 
     case OP_STRING:
       tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+      (*pos) += 2 + ((tem + sizeof (union exp_element))
+		     / sizeof (union exp_element));
       if (noside == EVAL_SKIP)
 	goto nosideret;
       return value_string (&exp->elts[pc + 1].string, tem);
 
     case TERNOP_COND:
       /* Skip third and second args to evaluate the first one.  */
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (value_zerop (arg1))
 	{
-	  evaluate_subexp (0, exp, pos, EVAL_SKIP);
-	  return evaluate_subexp (0, exp, pos, noside);
+	  evaluate_subexp (NULL_TYPE, exp, pos, EVAL_SKIP);
+	  return evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	}
       else
 	{
-	  arg2 = evaluate_subexp (0, exp, pos, noside);
-	  evaluate_subexp (0, exp, pos, EVAL_SKIP);
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
+	  evaluate_subexp (NULL_TYPE, exp, pos, EVAL_SKIP);
 	  return arg2;
 	}
 
@@ -210,7 +246,6 @@ evaluate_subexp (expect_type, exp, pos, noside)
       if (op == STRUCTOP_MEMBER || op == STRUCTOP_MPTR)
 	{
 	  int fnptr;
-	  int tem2;
 
 	  nargs = (int) exp->elts[pc + 1].longconst + 1;
 	  /* First, evaluate the structure into arg2 */
@@ -225,7 +260,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	    }
 	  else
 	    {
-	      arg2 = evaluate_subexp (0, exp, pos, noside);
+	      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	    }
 
 	  /* If the function is a virtual function, then the
@@ -234,7 +269,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	     it is just along for the ride: call the function
 	     directly.  */
 
-	  arg1 = evaluate_subexp (0, exp, pos, noside);
+	  arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 
 	  fnptr = (int) value_as_long (arg1);
 	  if (fnptr < 128)
@@ -299,7 +334,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	    }
 	  else
 	    {
-	      arg2 = evaluate_subexp (0, exp, pos, noside);
+	      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	    }
 	  /* Now, say which argument to start evaluating from */
 	  tem = 2;
@@ -319,11 +354,28 @@ evaluate_subexp (expect_type, exp, pos, noside)
 
       if (op == STRUCTOP_STRUCT || op == STRUCTOP_PTR)
 	{
+	  int static_memfuncp;
+	  value temp = arg2;
+
 	  argvec[1] = arg2;
 	  argvec[0] =
-	    value_struct_elt (arg2, argvec+1, &exp->elts[pc2 + 1].string,
+	    value_struct_elt (&temp, argvec+1, &exp->elts[pc2 + 1].string,
+			      &static_memfuncp,
 			      op == STRUCTOP_STRUCT
 			      ? "structure" : "structure pointer");
+	  if (VALUE_OFFSET (temp))
+	    {
+	      arg2 = value_from_long (builtin_type_long,
+				      value_as_long (arg2)+VALUE_OFFSET (temp));
+	      VALUE_TYPE (arg2) = lookup_pointer_type (VALUE_TYPE (temp));
+	      argvec[1] = arg2;
+	    }
+	  if (static_memfuncp)
+	    {
+	      argvec[1] = argvec[0];
+	      nargs--;
+	      argvec++;
+	    }
 	}
       else if (op == STRUCTOP_MEMBER || op == STRUCTOP_MPTR)
 	{
@@ -350,50 +402,80 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	  else
 	    error ("Expression of type other than \"Function returning ...\" used as function");
 	}
-      return call_function (argvec[0], nargs, argvec + 1);
+      return target_call_function (argvec[0], nargs, argvec + 1);
 
     case STRUCTOP_STRUCT:
       tem = strlen (&exp->elts[pc + 1].string);
-      (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      (*pos) += 2 + ((tem + sizeof (union exp_element))
+		     / sizeof (union exp_element));
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      return value_struct_elt (arg1, 0, &exp->elts[pc + 1].string,
-			       "structure");
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	return value_zero (lookup_struct_elt_type (VALUE_TYPE (arg1),
+						   &exp->elts[pc + 1].string),
+			   lval_memory);
+      else
+	{
+	  value temp = arg1;
+	  return value_struct_elt (&temp, (value *)0, &exp->elts[pc + 1].string,
+				   (int *) 0, "structure");
+	}
 
     case STRUCTOP_PTR:
       tem = strlen (&exp->elts[pc + 1].string);
       (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      return value_struct_elt (arg1, 0, &exp->elts[pc + 1].string,
-			       "structure pointer");
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	return value_zero (lookup_struct_elt_type (TYPE_TARGET_TYPE
+						   (VALUE_TYPE (arg1)),
+						   &exp->elts[pc + 1].string),
+			   lval_memory);
+      else
+	{
+	  value temp = arg1;
+	  return value_struct_elt (&temp, (value *)0, &exp->elts[pc + 1].string,
+				   (int *) 0, "structure pointer");
+	}
 
     case STRUCTOP_MEMBER:
       arg1 = evaluate_subexp_for_address (exp, pos, noside);
-      arg2 = evaluate_subexp (0, exp, pos, noside);
+      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       /* Now, convert these values to an address.  */
+      if (TYPE_CODE (VALUE_TYPE (arg2)) != TYPE_CODE_PTR
+	  || ((TYPE_CODE (TYPE_TARGET_TYPE (VALUE_TYPE (arg2)))
+	       != TYPE_CODE_MEMBER)
+	      && (TYPE_CODE (TYPE_TARGET_TYPE (VALUE_TYPE (arg2)))
+		  != TYPE_CODE_METHOD)))
+	error ("non-pointer-to-member value used in pointer-to-member construct");
       arg3 = value_from_long (builtin_type_long,
 			      value_as_long (arg1) + value_as_long (arg2));
-      VALUE_TYPE (arg3) = lookup_pointer_type (TYPE_TARGET_TYPE (VALUE_TYPE (arg2)));
+      VALUE_TYPE (arg3) =
+	lookup_pointer_type (TYPE_TARGET_TYPE (TYPE_TARGET_TYPE (VALUE_TYPE (arg2))));
       return value_ind (arg3);
 
     case STRUCTOP_MPTR:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
-      arg2 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
+      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       /* Now, convert these values to an address.  */
+      if (TYPE_CODE (VALUE_TYPE (arg2)) != TYPE_CODE_PTR
+	  || (TYPE_CODE (TYPE_TARGET_TYPE (VALUE_TYPE (arg2))) != TYPE_CODE_MEMBER
+	      && TYPE_CODE (TYPE_TARGET_TYPE (VALUE_TYPE (arg2))) != TYPE_CODE_METHOD))
+	error ("non-pointer-to-member value used in pointer-to-member construct");
       arg3 = value_from_long (builtin_type_long,
 			      value_as_long (arg1) + value_as_long (arg2));
-      VALUE_TYPE (arg3) = lookup_pointer_type (TYPE_TARGET_TYPE (VALUE_TYPE (arg2)));
+      VALUE_TYPE (arg3) =
+	lookup_pointer_type (TYPE_TARGET_TYPE (TYPE_TARGET_TYPE (VALUE_TYPE (arg2))));
       return value_ind (arg3);
 
     case BINOP_ASSIGN:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP || noside == EVAL_AVOID_SIDE_EFFECTS)
 	return arg1;
@@ -404,7 +486,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 
     case BINOP_ASSIGN_MODIFY:
       (*pos) += 2;
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP || noside == EVAL_AVOID_SIDE_EFFECTS)
 	return arg1;
@@ -447,12 +529,16 @@ evaluate_subexp (expect_type, exp, pos, noside)
     case BINOP_LOGAND:
     case BINOP_LOGIOR:
     case BINOP_LOGXOR:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
-      arg2 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
+      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       if (binop_user_defined_p (op, arg1, arg2))
 	return value_x_binop (arg1, arg2, op, 0);
+      else
+	if (noside == EVAL_AVOID_SIDE_EFFECTS
+	    && op == BINOP_DIV)
+	  return value_zero (VALUE_TYPE (arg1), not_lval);
       else
 	return value_binop (arg1, arg2, op);
 
@@ -461,65 +547,69 @@ evaluate_subexp (expect_type, exp, pos, noside)
       arg2 = evaluate_subexp_with_coercion (exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	return value_zero (TYPE_TARGET_TYPE (VALUE_TYPE (arg1)),
+			   VALUE_LVAL (arg1));
+			   
       if (binop_user_defined_p (op, arg1, arg2))
 	return value_x_binop (arg1, arg2, op, 0);
       else
-	return value_subscript (arg1, arg2, op);
+	return value_subscript (arg1, arg2);
       
     case BINOP_AND:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	{
-	  arg2 = evaluate_subexp (0, exp, pos, noside);
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	  goto nosideret;
 	}
       
       oldpos = *pos;
-      arg2 = evaluate_subexp (0, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
+      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
       *pos = oldpos;
       
       if (binop_user_defined_p (op, arg1, arg2)) 
 	{
-	  arg2 = evaluate_subexp (0, exp, pos, noside);
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	  return value_x_binop (arg1, arg2, op, 0);
 	}
       else
 	{
 	  tem = value_zerop (arg1);
-	  arg2 = evaluate_subexp (0, exp, pos,
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos,
 				  (tem ? EVAL_SKIP : noside));
 	  return value_from_long (builtin_type_int,
 				  (LONGEST) (!tem && !value_zerop (arg2)));
 	}
 
     case BINOP_OR:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	{
-	  arg2 = evaluate_subexp (0, exp, pos, noside);
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	  goto nosideret;
 	}
       
       oldpos = *pos;
-      arg2 = evaluate_subexp (0, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
+      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
       *pos = oldpos;
       
       if (binop_user_defined_p (op, arg1, arg2)) 
 	{
-	  arg2 = evaluate_subexp (0, exp, pos, noside);
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
 	  return value_x_binop (arg1, arg2, op, 0);
 	}
       else
 	{
 	  tem = value_zerop (arg1);
-	  arg2 = evaluate_subexp (0, exp, pos,
+	  arg2 = evaluate_subexp (NULL_TYPE, exp, pos,
 				  (!tem ? EVAL_SKIP : noside));
 	  return value_from_long (builtin_type_int,
 				  (LONGEST) (!tem || !value_zerop (arg2)));
 	}
 
     case BINOP_EQUAL:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -534,7 +624,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
 
     case BINOP_NOTEQUAL:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -549,7 +639,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
 
     case BINOP_LESS:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -564,7 +654,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
 
     case BINOP_GTR:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -579,7 +669,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
 
     case BINOP_GEQ:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -594,7 +684,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
 
     case BINOP_LEQ:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       arg2 = evaluate_subexp (VALUE_TYPE (arg1), exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
@@ -609,40 +699,46 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	}
 
     case BINOP_REPEAT:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
-      arg2 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
+      arg2 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      return value_repeat (arg1, (int) value_as_long (arg2));
+      if (TYPE_CODE (VALUE_TYPE (arg2)) != TYPE_CODE_INT)
+	error ("Non-integral right operand for \"@\" operator.");
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	return allocate_repeat_value (VALUE_TYPE (arg1),
+				       (int) value_as_long (arg2));
+      else
+	return value_repeat (arg1, (int) value_as_long (arg2));
 
     case BINOP_COMMA:
-      evaluate_subexp (0, exp, pos, noside);
-      return evaluate_subexp (0, exp, pos, noside);
+      evaluate_subexp (NULL_TYPE, exp, pos, noside);
+      return evaluate_subexp (NULL_TYPE, exp, pos, noside);
 
     case UNOP_NEG:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       if (unop_user_defined_p (op, arg1))
-	return value_x_unop (arg1, op, 0);
+	return value_x_unop (arg1, op);
       else
 	return value_neg (arg1);
 
     case UNOP_LOGNOT:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       if (unop_user_defined_p (op, arg1))
-	return value_x_unop (arg1, op, 0);
+	return value_x_unop (arg1, op);
       else
 	return value_lognot (arg1);
 
     case UNOP_ZEROP:
-      arg1 = evaluate_subexp (0, exp, pos, noside);
+      arg1 = evaluate_subexp (NULL_TYPE, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
       if (unop_user_defined_p (op, arg1))
-	return value_x_unop (arg1, op, 0);
+	return value_x_unop (arg1, op);
       else
 	return value_from_long (builtin_type_int,
 				(LONGEST) value_zerop (arg1));
@@ -653,6 +749,21 @@ evaluate_subexp (expect_type, exp, pos, noside)
       arg1 = evaluate_subexp (expect_type, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	{
+	  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_PTR
+	      || TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_REF
+	      /* In C you can dereference an array to get the 1st elt.  */
+	      || TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_ARRAY
+	      )
+	    return value_zero (TYPE_TARGET_TYPE (VALUE_TYPE (arg1)),
+			       lval_memory);
+	  else if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_INT)
+	    /* GDB allows dereferencing an int.  */
+	    return value_zero (builtin_type_int, lval_memory);
+	  else
+	    error ("Attempt to take contents of a non-pointer value.");
+	}
       return value_ind (arg1);
 
     case UNOP_ADDR:
@@ -665,8 +776,8 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	  if (op == OP_SCOPE)
 	    {
 	      char *name = &exp->elts[pc+3].string;
-	      int tem = strlen (name);
-	      (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+	      int temm = strlen (name);
+	      (*pos) += 2 + (temm + sizeof (union exp_element)) / sizeof (union exp_element);
 	    }
 	  else
 	    evaluate_subexp (expect_type, exp, pos, EVAL_SKIP);
@@ -676,9 +787,9 @@ evaluate_subexp (expect_type, exp, pos, noside)
       if (op == OP_SCOPE)
 	{
 	  char *name = &exp->elts[pc+3].string;
-	  int tem = strlen (name);
+	  int temm = strlen (name);
 	  struct type *domain = exp->elts[pc+2].type;
-	  (*pos) += 2 + (tem + sizeof (union exp_element)) / sizeof (union exp_element);
+	  (*pos) += 2 + (temm + sizeof (union exp_element)) / sizeof (union exp_element);
 	  arg1 = value_struct_elt_for_address (domain, expect_type, name);
 	  if (arg1)
 	    return arg1;
@@ -690,7 +801,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
     case UNOP_SIZEOF:
       if (noside == EVAL_SKIP)
 	{
-	  evaluate_subexp (0, exp, pos, EVAL_SKIP);
+	  evaluate_subexp (NULL_TYPE, exp, pos, EVAL_SKIP);
 	  goto nosideret;
 	}
       return evaluate_subexp_for_sizeof (exp, pos);
@@ -707,8 +818,11 @@ evaluate_subexp (expect_type, exp, pos, noside)
       arg1 = evaluate_subexp (expect_type, exp, pos, noside);
       if (noside == EVAL_SKIP)
 	goto nosideret;
-      return value_at (exp->elts[pc + 1].type,
-		       (CORE_ADDR) value_as_long (arg1));
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	return value_zero (exp->elts[pc + 1].type, lval_memory);
+      else
+	return value_at_lazy (exp->elts[pc + 1].type,
+			 (CORE_ADDR) value_as_long (arg1));
 
     case UNOP_PREINCREMENT:
       arg1 = evaluate_subexp (expect_type, exp, pos, noside);
@@ -716,7 +830,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	return arg1;
       else if (unop_user_defined_p (op, arg1))
 	{
-	  return value_x_unop (arg1, op, 0);
+	  return value_x_unop (arg1, op);
 	}
       else
 	{
@@ -731,7 +845,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	return arg1;
       else if (unop_user_defined_p (op, arg1))
 	{
-	  return value_x_unop (arg1, op, 0);
+	  return value_x_unop (arg1, op);
 	}
       else
 	{
@@ -746,7 +860,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	return arg1;
       else if (unop_user_defined_p (op, arg1))
 	{
-	  return value_x_unop (arg1, op, 0);
+	  return value_x_unop (arg1, op);
 	}
       else
 	{
@@ -762,7 +876,7 @@ evaluate_subexp (expect_type, exp, pos, noside)
 	return arg1;
       else if (unop_user_defined_p (op, arg1))
 	{
-	  return value_x_unop (arg1, op, 0);
+	  return value_x_unop (arg1, op);
 	}
       else
 	{
@@ -807,19 +921,45 @@ evaluate_subexp_for_address (exp, pos, noside)
     {
     case UNOP_IND:
       (*pos)++;
-      return evaluate_subexp (0, exp, pos, noside);
+      return evaluate_subexp (NULL_TYPE, exp, pos, noside);
 
     case UNOP_MEMVAL:
       (*pos) += 3;
       return value_cast (lookup_pointer_type (exp->elts[pc + 1].type),
-			 evaluate_subexp (0, exp, pos, noside));
+			 evaluate_subexp (NULL_TYPE, exp, pos, noside));
 
     case OP_VAR_VALUE:
       (*pos) += 3;
-      return locate_var_value (exp->elts[pc + 1].symbol, (CORE_ADDR) 0);
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	{
+	  struct type *type =
+	    lookup_pointer_type (SYMBOL_TYPE (exp->elts[pc + 1].symbol));
+	  enum address_class sym_class =
+	    SYMBOL_CLASS (exp->elts[pc + 1].symbol);
+
+	  if (sym_class == LOC_CONST
+	      || sym_class == LOC_CONST_BYTES
+	      || sym_class == LOC_REGISTER
+	      || sym_class == LOC_REGPARM)
+	    error ("Attempt to take address of register or constant.");
+
+	return
+	  value_zero (type, not_lval);
+	}
+      else
+	return locate_var_value (exp->elts[pc + 1].symbol, (CORE_ADDR) 0);
 
     default:
-      return value_addr (evaluate_subexp (0, exp, pos, noside));
+      if (noside == EVAL_AVOID_SIDE_EFFECTS)
+	{
+	  value x = evaluate_subexp (NULL_TYPE, exp, pos, noside);
+	  if (VALUE_LVAL (x) == lval_memory)
+	    return value_zero (TYPE_POINTER_TYPE (VALUE_TYPE (x)),
+			       not_lval);
+	  else
+	    error ("Attempt to take address of non-lval");
+	}
+      return value_addr (evaluate_subexp (NULL_TYPE, exp, pos, noside));
     }
 }
 
@@ -851,9 +991,9 @@ evaluate_subexp_with_coercion (exp, pos, noside)
 	  return value_cast (lookup_pointer_type (TYPE_TARGET_TYPE (SYMBOL_TYPE (exp->elts[pc + 1].symbol))),
 			     val);
 	}
+      default:
+	return evaluate_subexp (NULL_TYPE, exp, pos, noside);
     }
-
-  return evaluate_subexp (0, exp, pos, noside);
 }
 
 /* Evaluate a subexpression of EXP, at index *POS,
@@ -880,7 +1020,7 @@ evaluate_subexp_for_sizeof (exp, pos)
 	 create a value unnecessarily.  */
     case UNOP_IND:
       (*pos)++;
-      val = evaluate_subexp (0, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
+      val = evaluate_subexp (NULL_TYPE, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
       return value_from_long (builtin_type_int, (LONGEST)
 		      TYPE_LENGTH (TYPE_TARGET_TYPE (VALUE_TYPE (val))));
 
@@ -895,7 +1035,7 @@ evaluate_subexp_for_sizeof (exp, pos)
 	 (LONGEST) TYPE_LENGTH (SYMBOL_TYPE (exp->elts[pc + 1].symbol)));
 
     default:
-      val = evaluate_subexp (0, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
+      val = evaluate_subexp (NULL_TYPE, exp, pos, EVAL_AVOID_SIDE_EFFECTS);
       return value_from_long (builtin_type_int,
 			      (LONGEST) TYPE_LENGTH (VALUE_TYPE (val)));
     }
