@@ -1,23 +1,26 @@
 /* Memory-access and commands for inferior process, for GDB.
-   Copyright (C) 1986, 1987, 1988 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1987, 1988, 1989 Free Software Foundation, Inc.
 
-GDB is distributed in the hope that it will be useful, but WITHOUT ANY
-WARRANTY.  No author or distributor accepts responsibility to anyone
-for the consequences of using it or for whether it serves any
-particular purpose or works at all, unless he says so in writing.
-Refer to the GDB General Public License for full details.
+This file is part of GDB.
 
-Everyone is granted permission to copy, modify and redistribute GDB,
-but only under the conditions described in the GDB General Public
-License.  A copy of this license is supposed to have been given to you
-along with GDB so you can know your rights and responsibilities.  It
-should be in a file named COPYING.  Among other things, the copyright
-notice and this notice must be preserved on all copies.
+GDB is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-In other words, go ahead and share GDB, but don't try to stop
-anyone else from sharing it farther.  Help stamp out software hoarding!
-*/
+GDB is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
+You should have received a copy of the GNU General Public License
+along with GDB; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+
+#include <stdio.h>
+#include <signal.h>
+#include <sys/param.h>
+#include <string.h>
 #include "defs.h"
 #include "param.h"
 #include "symtab.h"
@@ -25,18 +28,19 @@ anyone else from sharing it farther.  Help stamp out software hoarding!
 #include "inferior.h"
 #include "environ.h"
 #include "value.h"
-
-#include <stdio.h>
-#include <signal.h>
-#include <sys/param.h>
+#include "gdbcmd.h"
+#include "gdbcore.h"
+#include "target.h"
 
 extern char *sys_siglist[];
 
+extern void until_break_command ();	/* breakpoint.c */
+
 #define ERROR_NO_INFERIOR \
    if (inferior_pid == 0) error ("The program is not being run.");
 
-/* String containing arguments to give to the program,
-   with a space added at the front.  Just a space means no args.  */
+/* String containing arguments to give to the program, separated by spaces.
+   Empty string (pointer to '\0') means no args.  */
 
 static char *inferior_args;
 
@@ -44,7 +48,10 @@ static char *inferior_args;
 
 char *inferior_io_terminal;
 
-/* Pid of our debugged inferior, or 0 if no inferior now.  */
+/* Pid of our debugged inferior, or 0 if no inferior now.
+   Since various parts of infrun.c test this to see whether there is a program
+   being debugged it should be nonzero (currently 3 is used) for remote
+   debugging.  */
 
 int inferior_pid;
 
@@ -60,9 +67,14 @@ CORE_ADDR stop_pc;
 
 FRAME_ADDR stop_frame_address;
 
-/* Number of breakpoint it stopped at, or 0 if none.  */
+/* Chain containing status of breakpoint(s) that we have stopped at.  */
 
-int stop_breakpoint;
+bpstat stop_bpstat;
+
+/* Flag indicating that a command has proceeded the inferior past the
+   current breakpoint.  */
+
+int breakpoint_proceeded;
 
 /* Nonzero if stopped due to a step command.  */
 
@@ -107,7 +119,7 @@ int step_multi;
 struct environ *inferior_environ;
 
 CORE_ADDR read_pc ();
-struct command_line *get_breakpoint_commands ();
+void breakpoint_clear_ignore_counts ();
 
 
 int
@@ -116,18 +128,10 @@ have_inferior_p ()
   return inferior_pid != 0;
 }
 
-static void 
-set_args_command (args)
-     char *args;
-{
-  free (inferior_args);
-  if (!args) args = "";
-  inferior_args = concat (" ", args, "");
-}
-
 void
-tty_command (file)
+tty_command (file, from_tty)
      char *file;
+     int from_tty;
 {
   if (file == 0)
     error_no_arg ("terminal name for running target process");
@@ -140,14 +144,7 @@ run_command (args, from_tty)
      char *args;
      int from_tty;
 {
-  extern char **environ;
-  register int i;
   char *exec_file;
-  char *allargs;
-
-  extern int sys_nerr;
-  extern char *sys_errlist[];
-  extern int errno;
 
   dont_repeat ();
 
@@ -157,72 +154,80 @@ run_command (args, from_tty)
 	  !query ("The program being debugged has been started already.\n\
 Start it from the beginning? "))
 	error ("Program not restarted.");
-      kill_inferior ();
+      target_kill ((char *)0, 0);
     }
 
-  if (remote_debugging)
+  exec_file = (char *) get_exec_file (0);
+
+  /* The exec file is re-read every time we do an inferior_died, so
+     we just have to worry about the symbol file.  */
+  reread_symbols ();
+
+  if (args)
     {
-      free (allargs);
-      if (from_tty)
-	{
-	  printf ("Starting program: %s%s\n",
-		  exec_file, inferior_args);
-	  fflush (stdout);
-	}
+      char *cmd;
+      cmd = concat ("set args ", args, "");
+      make_cleanup (free, cmd);
+      execute_command (cmd, from_tty);
     }
-  else
+
+  if (from_tty)
     {
-      if (args)
-	set_args_command (args);
-
-      exec_file = (char *) get_exec_file (1);
-      if (from_tty)
-	{
-	  printf ("Starting program: %s%s\n",
-		  exec_file, inferior_args);
-	  fflush (stdout);
-	}
-
-      allargs = concat ("exec ", exec_file, inferior_args);
-      inferior_pid = create_inferior (allargs, environ_vector (inferior_environ));
+      printf ("Starting program: %s %s\n",
+	      exec_file? exec_file: "", inferior_args);
+      fflush (stdout);
     }
 
-  clear_proceed_status ();
-
-  start_inferior ();
+  target_create_inferior (exec_file, inferior_args,
+			  environ_vector (inferior_environ));
 }
 
 void
-cont_command (proc_count_exp, from_tty)
+continue_command (proc_count_exp, from_tty)
      char *proc_count_exp;
      int from_tty;
 {
   ERROR_NO_INFERIOR;
 
-  clear_proceed_status ();
-
   /* If have argument, set proceed count of breakpoint we stopped at.  */
 
-  if (stop_breakpoint > 0 && proc_count_exp)
+  if (proc_count_exp != NULL)
     {
-      set_ignore_count (stop_breakpoint,
-			parse_and_eval_address (proc_count_exp) - 1,
-			from_tty);
-      if (from_tty)
-	printf ("  ");
+      bpstat bs = stop_bpstat;
+      int num = bpstat_num (&bs);
+      if (num == 0 && from_tty)
+	{
+	  printf_filtered
+	    ("Not stopped at any breakpoint; argument ignored.\n");
+	}
+      while (num != 0)
+	{
+	  set_ignore_count (num,
+			    parse_and_eval_address (proc_count_exp) - 1,
+			    from_tty);
+	  /* set_ignore_count prints a message ending with a period.
+	     So print two spaces before "Continuing.".  */
+	  if (from_tty)
+	    printf ("  ");
+	  num = bpstat_num (&bs);
+	}
     }
 
   if (from_tty)
     printf ("Continuing.\n");
 
-  proceed (-1, -1, 0);
+  clear_proceed_status ();
+
+  proceed ((CORE_ADDR) -1, -1, 0);
 }
 
 /* Step until outside of current statement.  */
 static void step_1 ();
 
 static void
-step_command (count_string)
+step_command (count_string, from_tty)
+     char * count_string;
+     int from_tty;
 {
   step_1 (0, 0, count_string);
 }
@@ -230,7 +235,9 @@ step_command (count_string)
 /* Likewise, but skip over subroutine calls as if single instructions.  */
 
 static void
-next_command (count_string)
+next_command (count_string, from_tty)
+     char * count_string;
+     int from_tty;
 {
   step_1 (1, 0, count_string);
 }
@@ -238,13 +245,17 @@ next_command (count_string)
 /* Likewise, but step only one instruction.  */
 
 static void
-stepi_command (count_string)
+stepi_command (count_string, from_tty)
+     char * count_string;
+     int from_tty;
 {
   step_1 (0, 1, count_string);
 }
 
 static void
-nexti_command (count_string)
+nexti_command (count_string, from_tty)
+     char * count_string;
+     int from_tty;
 {
   step_1 (1, 1, count_string);
 }
@@ -274,7 +285,7 @@ step_1 (skip_subroutines, single_inst, count_string)
 	      int misc;
 
 	      misc = find_pc_misc_function (stop_pc);
-	      terminal_ours ();
+	      target_terminal_ours ();
 	      printf ("Current function has no line number information.\n");
 	      fflush (stdout);
 
@@ -303,9 +314,13 @@ step_1 (skip_subroutines, single_inst, count_string)
 	step_over_calls = 1;
 
       step_multi = (count > 1);
-      proceed (-1, -1, 1);
+      proceed ((CORE_ADDR) -1, -1, 1);
       if (! stop_step)
 	break;
+#if defined (SHIFT_INST_REGS)
+      write_register (NNPC_REGNUM, read_register (NPC_REGNUM));
+      write_register (NPC_REGNUM, read_register (PC_REGNUM));
+#endif
     }
 }
 
@@ -352,13 +367,12 @@ jump_command (arg, from_tty)
   if (sal.pc == 0)
     error ("No line %d in file \"%s\".", sal.line, sal.symtab->filename);
 
-  addr = sal.pc;
-
-  clear_proceed_status ();
+  addr = ADDR_BITS_SET (sal.pc);
 
   if (from_tty)
     printf ("Continuing at 0x%x.\n", addr);
 
+  clear_proceed_status ();
   proceed (addr, 0, 0);
 }
 
@@ -379,11 +393,10 @@ signal_command (signum_exp, from_tty)
 
   signum = parse_and_eval_address (signum_exp);
 
-  clear_proceed_status ();
-
   if (from_tty)
     printf ("Continuing with signal %d.\n", signum);
 
+  clear_proceed_status ();
   proceed (stop_pc, signum, 0);
 }
 
@@ -409,7 +422,7 @@ static int stack_dummy_testing = 0;
 void
 run_stack_dummy (addr, buffer)
      CORE_ADDR addr;
-     REGISTER_TYPE *buffer;
+     char buffer[REGISTER_BYTES];
 {
   /* Now proceed, having reached the desired place.  */
   clear_proceed_status ();
@@ -421,21 +434,19 @@ run_stack_dummy (addr, buffer)
   proceed (addr, 0, 0);
 
   if (!stop_stack_dummy)
-    error ("Cannot continue previously requested operation.");
+    /* This used to say
+       "Cannot continue previously requested operation".  */
+    error ("\
+The program being debugged stopped while in a function called from GDB.\n\
+The expression which contained the function call has been discarded.");
 
   /* On return, the stack dummy has been popped already.  */
 
   bcopy (stop_registers, buffer, sizeof stop_registers);
 }
 
-/* Proceed until we reach the given line as argument or exit the
-   function.  When called with no argument, proceed until we reach a
-   different source line with pc greater than our current one or exit
-   the function.  We skip calls in both cases.
-
-   The effect of this command with an argument is identical to setting
-   a momentary breakpoint at the line specified and executing
-   "finish".
+/* Proceed until we reach a different source line with pc greater than
+   our current one or exit the function.  We skip calls in both cases.
 
    Note that eventually this command should probably be changed so
    that only source lines are printed out when we hit the breakpoint
@@ -443,15 +454,14 @@ run_stack_dummy (addr, buffer)
    of wait_for_inferior and the proceed status code. -- randy */
 
 void
-until_next_command (arg, from_tty)
-     char *arg;
+until_next_command (from_tty)
      int from_tty;
 {
   FRAME frame;
   CORE_ADDR pc;
   struct symbol *func;
   struct symtab_and_line sal;
-    
+ 
   clear_proceed_status ();
 
   frame = get_current_frame ();
@@ -486,7 +496,7 @@ until_next_command (arg, from_tty)
   
   step_multi = 0;		/* Only one call to proceed */
   
-  proceed (-1, -1, 1);
+  proceed ((CORE_ADDR) -1, -1, 1);
 }
 
 void 
@@ -494,13 +504,12 @@ until_command (arg, from_tty)
      char *arg;
      int from_tty;
 {
-  if (!have_inferior_p ())
-    error ("The program is not being run.");
-
+  if (!target_has_execution)
+    error ("The program is not running.");
   if (arg)
     until_break_command (arg, from_tty);
   else
-    until_next_command (arg, from_tty);
+    until_next_command (from_tty);
 }
 
 /* "finish": Set a temporary breakpoint at the place
@@ -516,10 +525,10 @@ finish_command (arg, from_tty)
   struct frame_info *fi;
   register struct symbol *function;
 
-  if (!have_inferior_p ())
-    error ("The program is not being run.");
   if (arg)
     error ("The \"finish\" command does not take any arguments.");
+  if (!target_has_execution)
+    error ("The program is not running.");
 
   frame = get_prev_frame (selected_frame);
   if (frame == 0)
@@ -543,9 +552,9 @@ finish_command (arg, from_tty)
       print_selected_frame ();
     }
 
-  proceed (-1, -1, 0);
+  proceed ((CORE_ADDR) -1, -1, 0);
 
-  if (stop_breakpoint == -3 && function != 0)
+  if (bpstat_momentary_breakpoint (stop_bpstat) && function != 0)
     {
       struct type *value_type;
       register value val;
@@ -561,12 +570,13 @@ finish_command (arg, from_tty)
       funcaddr = BLOCK_START (SYMBOL_BLOCK_VALUE (function));
 
       val = value_being_returned (value_type, stop_registers,
-				  using_struct_return (function,
-						       funcaddr,
-						       value_type));
+	      using_struct_return (value_of_variable (function),
+				   funcaddr,
+				   value_type,
+		BLOCK_GCC_COMPILED (SYMBOL_BLOCK_VALUE (function))));
 
       printf ("Value returned is $%d = ", record_latest_value (val));
-      value_print (val, stdout, 0);
+      value_print (val, stdout, 0, Val_no_prettyprint);
       putchar ('\n');
     }
 }
@@ -574,6 +584,9 @@ finish_command (arg, from_tty)
 static void
 program_info ()
 {
+  bpstat bs = stop_bpstat;
+  int num = bpstat_num (&bs);
+  
   if (inferior_pid == 0)
     {
       printf ("The program being debugged is not being run.\n");
@@ -584,13 +597,25 @@ program_info ()
 	  inferior_pid, stop_pc);
   if (stop_step)
     printf ("It stopped after being stepped.\n");
-  else if (stop_breakpoint > 0)
-    printf ("It stopped at breakpoint %d.\n", stop_breakpoint);
+  else if (num != 0)
+    {
+      /* There may be several breakpoints in the same place, so this
+	 isn't as strange as it seems.  */
+      while (num != 0)
+	{
+	  if (num < 0)
+	    printf ("It stopped at a breakpoint that has since been deleted.\n");
+	  else
+	    printf ("It stopped at breakpoint %d.\n", num);
+	  num = bpstat_num (&bs);
+	}
+    }
   else if (stop_signal)
     printf ("It stopped with signal %d (%s).\n",
 	    stop_signal, sys_siglist[stop_signal]);
 
-  printf ("\nType \"info stack\" or \"info reg\" for more information.\n");
+  printf (
+"\nType \"info stack\" or \"info registers\" for more information.\n");
 }
 
 static void
@@ -624,8 +649,8 @@ set_environment_command (arg)
     error_no_arg ("environment variable and value");
 
   /* Find seperation between variable name and value */
-  p = (char *) index (arg, '=');
-  val = (char *) index (arg, ' ');
+  p = (char *) strchr (arg, '=');
+  val = (char *) strchr (arg, ' ');
 
   if (p != 0 && val != 0)
     {
@@ -674,54 +699,28 @@ set_environment_command (arg)
 }
 
 static void
-unset_environment_command (var)
+unset_environment_command (var, from_tty)
      char *var;
+     int from_tty;
 {
   if (var == 0)
-    error_no_arg ("environment variable");
-
-  unset_in_environ (inferior_environ, var);
-}
-
-/* Read an integer from debugged memory, given address and number of bytes.  */
-
-long
-read_memory_integer (memaddr, len)
-     CORE_ADDR memaddr;
-     int len;
-{
-  char cbuf;
-  short sbuf;
-  int ibuf;
-  long lbuf;
-
-  if (len == sizeof (char))
     {
-      read_memory (memaddr, &cbuf, len);
-      return cbuf;
+      /* If there is no argument, delete all environment variables.
+	 Ask for confirmation if reading from the terminal.  */
+      if (!from_tty || query ("Delete all environment variables? "))
+	{
+	  free_environ (inferior_environ);
+	  inferior_environ = make_environ ();
+	}
     }
-  if (len == sizeof (short))
-    {
-      read_memory (memaddr, &sbuf, len);
-      return sbuf;
-    }
-  if (len == sizeof (int))
-    {
-      read_memory (memaddr, &ibuf, len);
-      return ibuf;
-    }
-  if (len == sizeof (lbuf))
-    {
-      read_memory (memaddr, &lbuf, len);
-      return lbuf;
-    }
-  error ("Cannot handle integers of %d bytes.", len);
+  else
+    unset_in_environ (inferior_environ, var);
 }
 
 CORE_ADDR
 read_pc ()
 {
-  return (CORE_ADDR) read_register (PC_REGNUM);
+  return ADDR_BITS_REMOVE ((CORE_ADDR) read_register (PC_REGNUM));
 }
 
 void
@@ -732,19 +731,102 @@ write_pc (val)
 #ifdef NPC_REGNUM
   write_register (NPC_REGNUM, (long) val+4);
 #endif
+  pc_changed = 0;
 }
 
 char *reg_names[] = REGISTER_NAMES;
 
+/* Print out the machine register regnum. If regnum is -1,
+   print all registers.
+   For most machines, having all_registers_info() print the
+   register(s) one per line is good enough. If a different format
+   is required, (eg, for SPARC or Pyramid 90x, which both have
+   lots of regs), or there is an existing convention for showing
+   all the registers, define the macro DO_REGISTERS_INFO(regnum)
+   to provide that format.  */  
+#if !defined (DO_REGISTERS_INFO)
+#define DO_REGISTERS_INFO(regnum) do_registers_info(regnum)
+static void do_registers_info (regnum)
+     int regnum;
+{
+  register int i;
+
+  if (regnum == -1)
+    printf_filtered (
+      "Register       Contents (relative to selected stack frame)\n\n");
+
+  for (i = 0; i < NUM_REGS; i++)
+    {
+      char raw_buffer[MAX_REGISTER_RAW_SIZE];
+      char virtual_buffer[MAX_REGISTER_VIRTUAL_SIZE];
+
+      if (regnum != -1 && i != regnum)
+	continue;
+
+      fputs_filtered (reg_names[i], stdout);
+      print_spaces_filtered (15 - strlen (reg_names[i]), stdout);
+
+      /* Get the data in raw format, then convert also to virtual format.  */
+      if (read_relative_register_raw_bytes (i, raw_buffer))
+	{
+	  printf_filtered ("Invalid register contents\n");
+	  continue;
+	}
+      
+      target_convert_to_virtual (i, raw_buffer, virtual_buffer);
+
+      /* If virtual format is floating, print it that way.  */
+      if (TYPE_CODE (REGISTER_VIRTUAL_TYPE (i)) == TYPE_CODE_FLT
+	  && ! INVALID_FLOAT (virtual_buffer, REGISTER_VIRTUAL_SIZE (i)))
+	val_print (REGISTER_VIRTUAL_TYPE (i), virtual_buffer, 0,
+		   stdout, 0, 1, 0, Val_pretty_default);
+
+/* FIXME!  val_print probably can handle all of these cases now...  */
+
+      /* Else if virtual format is too long for printf,
+	 print in hex a byte at a time.  */
+      else if (REGISTER_VIRTUAL_SIZE (i) > sizeof (long))
+	{
+	  register int j;
+	  printf_filtered ("0x");
+	  for (j = 0; j < REGISTER_VIRTUAL_SIZE (i); j++)
+	    printf_filtered ("%02x", (unsigned char)virtual_buffer[j]);
+	}
+      /* Else print as integer in hex and in decimal.  */
+      else
+	{
+	  val_print (REGISTER_VIRTUAL_TYPE (i), raw_buffer, 0,
+		     stdout, 'x', 1, 0, Val_pretty_default);
+	  printf_filtered ("\t");
+	  val_print (REGISTER_VIRTUAL_TYPE (i), raw_buffer, 0,
+		     stdout,   0, 1, 0, Val_pretty_default);
+	}
+
+      /* If register has different raw and virtual formats,
+	 print the raw format in hex now.  */
+
+      if (REGISTER_CONVERTIBLE (i))
+	{
+	  register int j;
+
+	  printf_filtered ("  (raw 0x");
+	  for (j = 0; j < REGISTER_RAW_SIZE (i); j++)
+	    printf_filtered ("%02x", (unsigned char)raw_buffer[j]);
+	  printf_filtered (")");
+	}
+      printf_filtered ("\n");
+    }
+}
+#endif /* no DO_REGISTERS_INFO.  */
+
 static void
 registers_info (addr_exp)
      char *addr_exp;
 {
-  register int i;
   int regnum;
 
-  if (!have_inferior_p () && !have_core_file_p ())
-    error ("No inferior or core file");
+  if (!target_has_registers)
+    error ("The program has no registers now.");
 
   if (addr_exp)
     {
@@ -763,77 +845,11 @@ registers_info (addr_exp)
 	}
     }
   else
-    printf ("Reg\tContents\n\n");
+    regnum = -1;
 
-  for (i = 0; i < NUM_REGS; i++)
-    {
-      unsigned char raw_buffer[MAX_REGISTER_RAW_SIZE];
-      unsigned char virtual_buffer[MAX_REGISTER_VIRTUAL_SIZE];
-      REGISTER_TYPE val;
-
-      if (addr_exp != 0 && i != regnum)
-	continue;
-
-      /* On machines with lots of registers, pause every 16 lines
-	 so user can read the output.  */
-      if (addr_exp == 0 && i > 0 && i % 16 == 0)
-	{
-	  printf ("--Type Return to print more--");
-	  fflush (stdout);
-	  gdb_read_line (0, 0);
-	}
-
-      /* Get the data in raw format, then convert also to virtual format.  */
-      read_relative_register_raw_bytes (i, raw_buffer);
-      REGISTER_CONVERT_TO_VIRTUAL (i, raw_buffer, virtual_buffer);
-
-      printf ("%s\t", reg_names[i]);
-
-      /* If virtual format is floating, print it that way.  */
-      if (TYPE_CODE (REGISTER_VIRTUAL_TYPE (i)) == TYPE_CODE_FLT
-	  && ! INVALID_FLOAT (virtual_buffer, REGISTER_VIRTUAL_SIZE (i)))
-	val_print (REGISTER_VIRTUAL_TYPE (i), virtual_buffer, 0,
-		   stdout, 0, 1);
-      /* Else if virtual format is too long for printf,
-	 print in hex a byte at a time.  */
-      else if (REGISTER_VIRTUAL_SIZE (i) > sizeof (long))
-	{
-	  register int j;
-	  printf ("0x");
-	  for (j = 0; j < REGISTER_VIRTUAL_SIZE (i); j++)
-	    printf ("%02x", virtual_buffer[j]);
-	}
-      /* Else print as integer in hex and in decimal.  */
-      else
-	{
-	  long val;
-
-	  bcopy (virtual_buffer, &val, sizeof (long));
-	  if (val == 0)
-	    printf ("0");
-	  else
-	    printf ("0x%08x  %d", val, val);
-	}
-
-      /* If register has different raw and virtual formats,
-	 print the raw format in hex now.  */
-
-      if (REGISTER_CONVERTIBLE (i))
-	{
-	  register int j;
-
-	  printf ("  (raw 0x");
-	  for (j = 0; j < REGISTER_RAW_SIZE (i); j++)
-	    printf ("%02x", raw_buffer[j]);
-	  printf (")");
-	}
-      printf ("\n");
-    }
-
-  printf ("Contents are relative to selected stack frame.\n");
+  DO_REGISTERS_INFO(regnum);
 }
 
-#ifdef ATTACH_DETACH
 /*
  * TODO:
  * Should save/restore the tty state since it might be that the
@@ -841,6 +857,7 @@ registers_info (addr_exp)
  * the tty in some state other than what we want.  If it's running
  * on another terminal or without a terminal, then saving and
  * restoring the tty state is a harmless no-op.
+ * This only needs to be done if we are attaching to a process.
  */
 
 /*
@@ -850,54 +867,12 @@ registers_info (addr_exp)
  * For this to work, we must be able to send the process a
  * signal and we must have the same effective uid as the program.
  */
-static void
+void
 attach_command (args, from_tty)
      char *args;
      int from_tty;
 {
-  char *exec_file;
-  int pid;
-  int remote = 0;
-
-  dont_repeat();
-
-  if (!args)
-    error_no_arg ("process-id to attach");
-
-  while (*args == ' ' || *args == '\t') args++;
-
-  if (args[0] == '/')
-    remote = 1;
-  else
-    pid = atoi (args);
-
-  if (inferior_pid)
-    {
-      if (query ("A program is being debugged already.  Kill it? "))
-	kill_inferior ();
-      else
-	error ("Inferior not killed.");
-    }
-
-  exec_file = (char *) get_exec_file (1);
-
-  if (from_tty)
-    {
-      if (remote)
-	printf ("Attaching remote machine\n");
-      else
-	printf ("Attaching program: %s pid %d\n",
-		exec_file, pid);
-      fflush (stdout);
-    }
-
-  if (remote)
-    {
-      remote_open (args, from_tty);
-      start_remote ();
-    }
-  else
-    attach_program (pid);
+  target_open (args, from_tty);
 }
 
 /*
@@ -916,28 +891,10 @@ detach_command (args, from_tty)
      char *args;
      int from_tty;
 {
-  int signal = 0;
-
-  if (!inferior_pid)
-    error ("Not currently tracing a program\n");
-  if (from_tty)
-    {
-      char *exec_file = (char *)get_exec_file (0);
-      if (exec_file == 0)
-	exec_file = "";
-      printf ("Detaching program: %s pid %d\n",
-	      exec_file, inferior_pid);
-      fflush (stdout);
-    }
-  if (args)
-    signal = atoi (args);
-
-  detach (signal);
-  inferior_pid = 0;
+  target_detach (args, from_tty);
 }
-#endif /* ATTACH_DETACH */
 
-/* ARGUSUED */
+/* ARGSUSED */
 static void
 float_info (addr_exp)
      char *addr_exp;
@@ -949,49 +906,58 @@ float_info (addr_exp)
 #endif
 }
 
-extern struct cmd_list_element *setlist, *deletelist;
-
 void
 _initialize_infcmd ()
 {
+  struct cmd_list_element *c;
+  
   add_com ("tty", class_run, tty_command,
 	   "Set terminal for future runs of program being debugged.");
 
-  add_cmd ("args", class_run, set_args_command,
-	   "Specify arguments to give program being debugged when it is started.\n\
+  add_show_from_set
+    (add_set_cmd ("args", class_run, var_string_noescape, (char *)&inferior_args,
+		  
+"Set arguments to give program being debugged when it is started.\n\
 Follow this command with any number of args, to be passed to the program.",
-	   &setlist);
+		  &setlist),
+     &showlist);
 
-  add_info ("environment", environment_info,
-	    "The environment to give the program, or one variable's value.\n\
+  c = add_cmd
+    ("environment", no_class, environment_info,
+     "The environment to give the program, or one variable's value.\n\
 With an argument VAR, prints the value of environment variable VAR to\n\
 give the program being debugged.  With no arguments, prints the entire\n\
-environment to be given to the program.");
+environment to be given to the program.", &showlist);
+  c->completer = noop_completer;
 
-  add_cmd ("environment", class_run, unset_environment_command,
-	   "Cancel environment variable VAR for the program.\n\
+  c = add_cmd ("environment", class_run, unset_environment_command,
+	      "Cancel environment variable VAR for the program.\n\
 This does not affect the program until the next \"run\" command.",
 	   &deletelist);
+  c->completer = noop_completer;
 
-  add_cmd ("environment", class_run, set_environment_command,
-	   "Set environment variable value to give the program.\n\
+  c = add_cmd ("environment", class_run, set_environment_command,
+	       "Set environment variable value to give the program.\n\
 Arguments are VAR VALUE where VAR is variable name and VALUE is value.\n\
 VALUES of environment variables are uninterpreted strings.\n\
 This does not affect the program until the next \"run\" command.",
 	   &setlist);
+  c->completer = noop_completer;
  
-#ifdef ATTACH_DETACH
  add_com ("attach", class_run, attach_command,
- 	   "Attach to a process that was started up outside of GDB.\n\
-To do this, you must have permission to send the process a signal.\n\
-And it must have the same effective uid as the debugger.\n\n\
-Before using \"attach\", you must use the \"exec-file\" command\n\
-to specify the program running in the process,\n\
-and the \"symbol-file\" command to load its symbol table.");
+ 	   "Attach to a process or file outside of GDB.\n\
+This command attaches to another target, of the same type as your last\n\
+`target' command (`info files' will show your target stack).\n\
+The command may take as argument a process id or a device file.\n\
+For a process id, you must have permission to send the process a signal,\n\
+and it must have the same effective uid as the debugger.\n\
+When using \"attach\", you should use the \"file\" command to specify\n\
+the program running in the process, and to load its symbol table.");
+
   add_com ("detach", class_run, detach_command,
-	   "Detach the process previously attached.\n\
-The process is no longer traced and continues its execution.");
-#endif /* ATTACH_DETACH */
+	   "Detach a process or file previously attached.\n\
+If a process, it is no longer traced, and it continues its execution.  If you\n\
+were debugging a file, the file is closed and gdb no longer accesses it.");
 
   add_com ("signal", class_run, signal_command,
 	   "Continue program giving it signal number SIGNUMBER.");
@@ -1033,19 +999,20 @@ Execution will also stop upon exit from the current stack frame.");
 Give as argument either LINENUM or *ADDR, where ADDR is an expression\n\
 for an address to start at.");
 
-  add_com ("cont", class_run, cont_command,
+  add_com ("continue", class_run, continue_command,
 	   "Continue program being debugged, after signal or breakpoint.\n\
 If proceeding from breakpoint, a number N may be used as an argument:\n\
 then the same breakpoint won't break until the Nth time it is reached.");
   add_com_alias ("c", "cont", class_run, 1);
+  add_com_alias ("fg", "cont", class_run, 1);
 
   add_com ("run", class_run, run_command,
 	   "Start debugged program.  You may specify arguments to give it.\n\
 Args may include \"*\", or \"[...]\"; they are expanded using \"sh\".\n\
 Input and output redirection with \">\", \"<\", or \">>\" are also allowed.\n\n\
-With no arguments, uses arguments last specified (with \"run\" or \"set-args\".\n\
+With no arguments, uses arguments last specified (with \"run\" or \"set args\".\n\
 To cancel previous arguments and run with no arguments,\n\
-use \"set-args\" without arguments.");
+use \"set args\" without arguments.");
   add_com_alias ("r", "run", class_run, 1);
 
   add_info ("registers", registers_info,
@@ -1058,8 +1025,7 @@ Register name as argument means describe only that register.");
   add_info ("float", float_info,
 	    "Print the status of the floating point unit\n");
 
-  inferior_args = savestring (" ", 1); /* By default, no args.  */
+  inferior_args = savestring ("", 1);	/* Initially no args */
   inferior_environ = make_environ ();
   init_environ (inferior_environ);
 }
-
