@@ -1,22 +1,21 @@
 /* Print and select stack frames for GDB, the GNU debugger.
    Copyright (C) 1986, 1987, 1989 Free Software Foundation, Inc.
 
-GDB is distributed in the hope that it will be useful, but WITHOUT ANY
-WARRANTY.  No author or distributor accepts responsibility to anyone
-for the consequences of using it or for whether it serves any
-particular purpose or works at all, unless he says so in writing.
-Refer to the GDB General Public License for full details.
+This file is part of GDB.
 
-Everyone is granted permission to copy, modify and redistribute GDB,
-but only under the conditions described in the GDB General Public
-License.  A copy of this license is supposed to have been given to you
-along with GDB so you can know your rights and responsibilities.  It
-should be in a file named COPYING.  Among other things, the copyright
-notice and this notice must be preserved on all copies.
+GDB is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-In other words, go ahead and share GDB, but don't try to stop
-anyone else from sharing it farther.  Help stamp out software hoarding!
-*/
+GDB is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GDB; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <stdio.h>
 
@@ -24,9 +23,15 @@ anyone else from sharing it farther.  Help stamp out software hoarding!
 #include "param.h"
 #include "symtab.h"
 #include "frame.h"
-#include "inferior.h"
+#include "gdbcmd.h"
+#include "value.h"
 #include "gdbcore.h"
+#include "target.h"
+#include "breakpoint.h"
 
+extern int addressprint;	/* Print addresses, or stay symbolic only? */
+extern int info_verbose;	/* Verbosity of symbol reading msgs */
+extern char *reg_names[];	/* Names of registers */
 
 /* Thie "selected" stack frame is used by default for local and arg access.
    May be zero, for no selected frame.  */
@@ -39,19 +44,14 @@ FRAME selected_frame;
 
 int selected_frame_level;
 
-/* Error message when selected_frame is zero when it's needed */
-char no_sel_frame[] = "There is no current stack frame.";
-
 /* Nonzero means print the full filename and linenumber
    when a frame is printed, and do so in a format programs can parse.  */
 
 int frame_file_full_name = 0;
 
-static void select_calling_frame ();
-
 void print_frame_info ();
 
-/* Print a stack frame briefly.  FRAME should be the frame address
+/* Print a stack frame briefly.  FRAME should be the frame id
    and LEVEL should be its level in the stack (or -1 for level not defined).
    This prints the level, the function executing, the arguments,
    and the file name and line number.
@@ -61,10 +61,6 @@ void print_frame_info ();
    If SOURCE is 1, print the source line as well.
    If SOURCE is -1, print ONLY the source line.  */
 
-/* FIXME, the argument "frame" is always "selected_frame".  This is why
-   we can say "No selected frame" if it == 0.  Probably shouldn't be an
-   argument anymore...  */
-
 static void
 print_stack_frame (frame, level, source)
      FRAME frame;
@@ -73,8 +69,6 @@ print_stack_frame (frame, level, source)
 {
   struct frame_info *fi;
 
-  if (frame == 0)
-    error (no_sel_frame);
   fi = get_frame_info (frame);
 
   print_frame_info (fi, level, source, 1);
@@ -92,10 +86,64 @@ print_frame_info (fi, level, source, args)
   register char *funname = 0;
   int numargs;
 
+#if 0	/* Symbol reading is fast enough now */
+  struct partial_symtab *pst;
+
+  /* Don't give very much information if we haven't readin the
+     symbol table yet.  */
+  pst = find_pc_psymtab (fi->pc);
+  if (pst && !pst->readin)
+    {
+      /* Abbreviated information.  */
+      char *fname;
+
+      if (!find_pc_partial_function (fi->pc, &fname, 0))
+	fname = "??";
+	
+      printf_filtered ("#%-2d ", level);
+      if (addressprint)
+        printf_filtered ("0x%x in ", fi->pc);
+
+      fputs_demangled (fname, stdout, -1);
+      fputs_filtered (" (...)\n", stdout);
+      
+      return;
+    }
+#endif
+
   sal = find_pc_line (fi->pc, fi->next_frame);
   func = find_pc_function (fi->pc);
   if (func)
-    funname = SYMBOL_NAME (func);
+    {
+      /* In certain pathological cases, the symtabs give the wrong
+	 function (when we are in the first function in a file which
+	 is compiled without debugging symbols, the previous function
+	 is compiled with debugging symbols, and the "foo.o" symbol
+	 that is supposed to tell us where the file with debugging symbols
+	 ends has been truncated by ar because it is longer than 15
+	 characters).
+
+	 So look in the misc_function_vector as well, and if it comes
+	 up with a larger address for the function use that instead.
+	 I don't think this can ever cause any problems;
+	 there shouldn't be any
+	 misc_function_vector symbols in the middle of a function.  */
+      int misc_index = find_pc_misc_function (fi->pc);
+      if (misc_index >= 0
+	  && (misc_function_vector[misc_index].address
+	      > BLOCK_START (SYMBOL_BLOCK_VALUE (func))))
+	{
+	  /* In this case we have no way of knowing the source file
+	     and line number, so don't print them.  */
+	  sal.symtab = 0;
+	  /* We also don't know anything about the function besides
+	     its address and name.  */
+	  func = 0;
+	  funname = misc_function_vector[misc_index].name;
+	}
+      else
+	funname = SYMBOL_NAME (func);
+    }
   else
     {
       register int misc_index = find_pc_misc_function (fi->pc);
@@ -106,22 +154,28 @@ print_frame_info (fi, level, source, args)
   if (source >= 0 || !sal.symtab)
     {
       if (level >= 0)
-	printf ("#%-2d ", level);
-      if (fi->pc != sal.pc || !sal.symtab)
-	printf ("0x%x in ", fi->pc);
-      printf ("%s (", funname ? funname : "??");
+	printf_filtered ("#%-2d ", level);
+      if (addressprint)
+	if (fi->pc != sal.pc || !sal.symtab)
+	  printf_filtered ("0x%x in ", fi->pc);
+      fputs_demangled (funname ? funname : "??", stdout, -1);
+      wrap_here ("   ");
+      fputs_filtered (" (", stdout);
       if (args)
 	{
 	  FRAME_NUM_ARGS (numargs, fi);
 	  print_frame_args (func, fi, numargs, stdout);
 	}
-      printf (")");
-      if (sal.symtab)
-	printf (" (%s line %d)", sal.symtab->filename, sal.line);
-      printf ("\n");
+      printf_filtered (")");
+      if (sal.symtab && sal.symtab->filename)
+	{
+          wrap_here ("   ");
+	  printf_filtered (" at %s:%d", sal.symtab->filename, sal.line);
+	}
+      printf_filtered ("\n");
     }
 
-  if (source != 0 && sal.symtab)
+  if ((source != 0) && sal.symtab)
     {
       int done = 0;
       int mid_statement = source < 0 && fi->pc != sal.pc;
@@ -129,11 +183,11 @@ print_frame_info (fi, level, source, args)
 	done = identify_source_line (sal.symtab, sal.line, mid_statement);
       if (!done)
 	{
-	  if (mid_statement)
-	    printf ("0x%x\t", fi->pc);
+	  if (addressprint && mid_statement)
+	    printf_filtered ("0x%x\t", fi->pc);
 	  print_source_lines (sal.symtab, sal.line, sal.line + 1, 1);
 	}
-      current_source_line = max (sal.line - 5, 1);
+      current_source_line = max (sal.line - lines_to_list () / 2, 1);
     }
   if (source != 0)
     set_default_breakpoint (1, fi->pc, sal.symtab, sal.line);
@@ -159,7 +213,7 @@ print_selected_frame ()
   print_stack_frame (selected_frame, selected_frame_level, 0);
 }
 
-void flush_cached_frames ();	/* FIXME, never called! */
+void flush_cached_frames ();
 
 #ifdef FRAME_SPECIFICATION_DYADIC
 extern FRAME setup_arbitrary_frame ();
@@ -179,7 +233,6 @@ parse_frame_specification (frame_exp)
     {
       char *addr_string, *p;
       struct cleanup *tmp_cleanup;
-      struct frame_info *fci;
 
       while (*frame_exp == ' ') frame_exp++;
       for (p = frame_exp; *p && *p != ' '; p++)
@@ -209,8 +262,6 @@ parse_frame_specification (frame_exp)
   switch (numargs)
     {
     case 0:
-      if (selected_frame == 0)
-	error (no_sel_frame);
       return selected_frame;
       /* NOTREACHED */
     case 1:
@@ -259,6 +310,16 @@ parse_frame_specification (frame_exp)
   /* NOTREACHED */
 }
 
+/* FRAME_ARGS_ADDRESS_CORRECT is just like FRAME_ARGS_ADDRESS except
+   that if it is unsure about the answer, it returns Frame_unknown
+   instead of guessing (this happens on the VAX, for example).
+
+   On most machines, we never have to guess about the args address,
+   so FRAME_ARGS_ADDRESS{,_CORRECT} are the same.  */
+#if !defined (FRAME_ARGS_ADDRESS_CORRECT)
+#define FRAME_ARGS_ADDRESS_CORRECT FRAME_ARGS_ADDRESS
+#endif
+
 /* Print verbosely the selected frame or the frame at address ADDR.
    This means absolutely all information in the frame is printed.  */
 
@@ -274,12 +335,15 @@ frame_info (addr_exp)
   FRAME calling_frame;
   int i, count;
   char *funname = 0;
-  int numargs;
+
+  if (!target_has_stack)
+    error ("No inferior or core file.");
 
   frame = parse_frame_specification (addr_exp);
+  if (!frame)
+    error ("Invalid frame specified.");
 
   fi = get_frame_info (frame);
-  get_frame_saved_regs (fi, &fsr);
   sal = find_pc_line (fi->pc, fi->next_frame);
   func = get_frame_function (frame);
   if (func)
@@ -293,17 +357,18 @@ frame_info (addr_exp)
   calling_frame = get_prev_frame (frame);
 
   if (!addr_exp && selected_frame_level >= 0)
-    printf ("Stack level %d, frame at 0x%x:\n pc = 0x%x",
-	    selected_frame_level, FRAME_FP(frame), fi->pc);
+    printf ("Stack level %d, frame at 0x%x:\n %s = 0x%x",
+	    selected_frame_level, FRAME_FP(frame),
+	    reg_names[PC_REGNUM], fi->pc);
   else
-    printf ("Stack frame at 0x%x:\n pc = 0x%x",
-	    FRAME_FP(frame), fi->pc);
+    printf ("Stack frame at 0x%x:\n %s = 0x%x",
+	    FRAME_FP(frame), reg_names[PC_REGNUM], fi->pc);
 
   if (funname)
     printf (" in %s", funname);
   if (sal.symtab)
     printf (" (%s line %d)", sal.symtab->filename, sal.line);
-  printf ("; saved pc 0x%x\n", FRAME_SAVED_PC (frame));
+  printf ("; saved %s 0x%x\n", reg_names[PC_REGNUM], FRAME_SAVED_PC (frame));
   if (calling_frame)
     printf (" called by frame at 0x%x", FRAME_FP (calling_frame));
   if (fi->next_frame && calling_frame)
@@ -312,23 +377,39 @@ frame_info (addr_exp)
     printf (" caller of frame at 0x%x", fi->next_frame);
   if (fi->next_frame || calling_frame)
     printf ("\n");
-  printf (" Arglist at 0x%x,", FRAME_ARGS_ADDRESS (fi));
-  FRAME_NUM_ARGS (i, fi);
-  if (i < 0)
-    printf (" args: ");
-  else if (i == 0)
-    printf (" no args.");
-  else if (i == 1)
-    printf (" 1 arg: ");
-  else
-    printf (" %d args: ", i);
 
-  FRAME_NUM_ARGS (numargs, fi);
-  print_frame_args (func, fi, numargs, stdout);
-  printf ("\n");
+  {
+    /* Address of the argument list for this frame, or Frame_unknown.  */
+    CORE_ADDR arg_list = FRAME_ARGS_ADDRESS_CORRECT (fi);
+    /* Number of args for this frame, or -1 if unknown.  */
+    int numargs;
+
+    if (arg_list != Frame_unknown)
+      {
+	printf (" Arglist at 0x%x,", arg_list);
+
+	FRAME_NUM_ARGS (numargs, fi);
+	if (numargs < 0)
+	  printf (" args: ");
+	else if (numargs == 0)
+	  printf (" no args.");
+	else if (numargs == 1)
+	  printf (" 1 arg: ");
+	else
+	  printf (" %d args: ", numargs);
+	print_frame_args (func, fi, numargs, stdout);
+	printf ("\n");
+      }
+  }
+
+#if defined (FRAME_FIND_SAVED_REGS)  
+  get_frame_saved_regs (fi, &fsr);
+  /* The sp is special; what's returned isn't the save address, but
+     actually the value of the previous frame's sp.  */
+  printf (" Previous frame's sp is 0x%x\n", fsr.regs[SP_REGNUM]);
   count = 0;
   for (i = 0; i < NUM_REGS; i++)
-    if (fsr.regs[i])
+    if (fsr.regs[i] && i != SP_REGNUM)
       {
 	if (count % 4 != 0)
 	  printf (", ");
@@ -343,6 +424,7 @@ frame_info (addr_exp)
       }
   if (count)
     printf ("\n");
+#endif /* Have FRAME_FIND_SAVED_REGS.  */
 }
 
 #if 0
@@ -379,8 +461,9 @@ backtrace_limit_info (arg, from_tty)
 /* Print briefly all stack frames or just the innermost COUNT frames.  */
 
 static void
-backtrace_command (count_exp)
+backtrace_command (count_exp, from_tty)
      char *count_exp;
+     int from_tty;
 {
   struct frame_info *fi;
   register int count;
@@ -389,9 +472,6 @@ backtrace_command (count_exp)
   register FRAME trailing;
   register int trailing_level;
 
-  if (have_inferior_p () == 0 && corefile == 0)
-    error ("There is no running program or core file.");
-
   /* The following code must do two things.  First, it must
      set the variable TRAILING to the frame from which we should start
      printing.  Second, it must set the variable count to the number
@@ -409,12 +489,16 @@ backtrace_command (count_exp)
 
 	  current = trailing;
 	  while (current && count--)
-	    current = get_prev_frame (current);
+	    {
+	      QUIT;
+	      current = get_prev_frame (current);
+	    }
 	  
 	  /* Will stop when CURRENT reaches the top of the stack.  TRAILING
 	     will be COUNT below it.  */
 	  while (current)
 	    {
+	      QUIT;
 	      trailing = get_prev_frame (trailing);
 	      current = get_prev_frame (current);
 	      trailing_level++;
@@ -424,11 +508,29 @@ backtrace_command (count_exp)
 	}
     }
   else
-#if 0    
-    count = backtrace_limit;
-#else
     count = -1;
-#endif  
+
+  if (info_verbose)
+    {
+      struct partial_symtab *ps;
+      
+      /* Read in symbols for all of the frames.  Need to do this in
+	 a separate pass so that "Reading in symbols for xxx" messages
+	 don't screw up the appearance of the backtrace.  Also
+	 if people have strong opinions against reading symbols for
+	 backtrace this may have to be an option.  */
+      i = count;
+      for (frame = trailing;
+	   frame != NULL && i--;
+	   frame = get_prev_frame (frame))
+	{
+	  QUIT;
+	  fi = get_frame_info (frame);
+	  ps = find_pc_psymtab (fi->pc);
+	  if (ps)
+	    (void) PSYMTAB_TO_SYMTAB (ps);	/* Force syms to come in */
+	}
+    }
 
   for (i = 0, frame = trailing;
        frame && count--;
@@ -440,13 +542,14 @@ backtrace_command (count_exp)
     }
 
   /* If we've stopped before the end, mention that.  */
-  if (frame)
-    printf ("(More stack frames follow...)\n");
+  if (frame && from_tty)
+    printf_filtered ("(More stack frames follow...)\n");
 }
 
-/* Print the local variables of a block B active in FRAME.  */
+/* Print the local variables of a block B active in FRAME.
+   Return 1 if any variables were printed; 0 otherwise.  */
 
-static void
+static int
 print_block_frame_locals (b, frame, stream)
      struct block *b;
      register FRAME frame;
@@ -455,6 +558,7 @@ print_block_frame_locals (b, frame, stream)
   int nsyms;
   register int i;
   register struct symbol *sym;
+  register int values_printed = 0;
 
   nsyms = BLOCK_NSYMS (b);
 
@@ -465,12 +569,57 @@ print_block_frame_locals (b, frame, stream)
 	  || SYMBOL_CLASS (sym) == LOC_REGISTER
 	  || SYMBOL_CLASS (sym) == LOC_STATIC)
 	{
-	  fprintf (stream, "%s = ", SYMBOL_NAME (sym));
+	  values_printed = 1;
+	  fprint_symbol (stream, SYMBOL_NAME (sym));
+	  fputs_filtered (" = ", stream);
 	  print_variable_value (sym, frame, stream);
-	  fprintf (stream, "\n");
+	  fprintf_filtered (stream, "\n");
 	  fflush (stream);
 	}
     }
+  return values_printed;
+}
+
+/* Same, but print labels.
+   FIXME, this does not even reference FRAME... --gnu  */
+
+static int
+print_block_frame_labels (b, frame, have_default, stream)
+     struct block *b;
+     register FRAME frame;
+     int *have_default;
+     register FILE *stream;
+{
+  int nsyms;
+  register int i;
+  register struct symbol *sym;
+  register int values_printed = 0;
+
+  nsyms = BLOCK_NSYMS (b);
+
+  for (i = 0; i < nsyms; i++)
+    {
+      sym = BLOCK_SYM (b, i);
+      if (! strcmp (SYMBOL_NAME (sym), "default"))
+	{
+	  if (*have_default)
+	    continue;
+	  *have_default = 1;
+	}
+      if (SYMBOL_CLASS (sym) == LOC_LABEL)
+	{
+	  struct symtab_and_line sal;
+	  sal = find_pc_line (SYMBOL_VALUE_ADDRESS (sym), 0);
+	  values_printed = 1;
+	  fputs_demangled (SYMBOL_NAME (sym), stream, 1);
+	  if (addressprint)
+	    fprintf_filtered (stream, " 0x%x", SYMBOL_VALUE_ADDRESS (sym));
+	  fprintf_filtered (stream, " in file %s, line %d\n",
+			    sal.symtab->filename, sal.line);
+	  fflush (stream);
+	}
+    }
+  return values_printed;
 }
 
 /* Print on STREAM all the local variables in frame FRAME,
@@ -486,14 +635,20 @@ print_frame_local_vars (frame, stream)
      register FRAME frame;
      register FILE *stream;
 {
-  register struct block *block;
+  register struct block *block = get_frame_block (frame);
+  register int values_printed = 0;
 
-  block = get_frame_block (frame);
   if (block == 0)
-    return 0;
+    {
+      fprintf_filtered (stream, "No symbol table info available.\n");
+      fflush (stream);
+      return 0;
+    }
+  
   while (block != 0)
     {
-      print_block_frame_locals (block, frame, stream);
+      if (print_block_frame_locals (block, frame, stream))
+	values_printed = 1;
       /* After handling the function's top-level block, stop.
 	 Don't continue to its superblock, the block of
 	 per-file symbols.  */
@@ -501,31 +656,132 @@ print_frame_local_vars (frame, stream)
 	break;
       block = BLOCK_SUPERBLOCK (block);
     }
+
+  if (!values_printed)
+    {
+      fprintf_filtered (stream, "No locals.\n");
+      fflush (stream);
+    }
+  
   return 1;
 }
 
-static void
-locals_info ()
+/* Same, but print labels.  */
+
+static int
+print_frame_label_vars (frame, this_level_only, stream)
+     register FRAME frame;
+     int this_level_only;
+     register FILE *stream;
 {
-  if (selected_frame == 0)
-    error(no_sel_frame);
+  extern struct blockvector *blockvector_for_pc ();
+  register struct blockvector *bl;
+  register struct block *block = get_frame_block (frame);
+  register int values_printed = 0;
+  int index, have_default = 0;
+  char *blocks_printed;
+  struct frame_info *fi = get_frame_info (frame);
+  CORE_ADDR pc = fi->pc;
+
+  if (block == 0)
+    {
+      fprintf_filtered (stream, "No symbol table info available.\n");
+      fflush (stream);
+      return 0;
+    }
+
+  bl = blockvector_for_pc (BLOCK_END (block) - 4, &index);
+  blocks_printed = (char *) alloca (BLOCKVECTOR_NBLOCKS (bl) * sizeof (char));
+  bzero (blocks_printed, BLOCKVECTOR_NBLOCKS (bl) * sizeof (char));
+
+  while (block != 0)
+    {
+      CORE_ADDR end = BLOCK_END (block) - 4;
+      int last_index;
+
+      if (bl != blockvector_for_pc (end, &index))
+	error ("blockvector blotch");
+      if (BLOCKVECTOR_BLOCK (bl, index) != block)
+	error ("blockvector botch");
+      last_index = BLOCKVECTOR_NBLOCKS (bl);
+      index += 1;
+
+      /* Don't print out blocks that have gone by.  */
+      while (index < last_index
+	     && BLOCK_END (BLOCKVECTOR_BLOCK (bl, index)) < pc)
+	index++;
+
+      while (index < last_index
+	     && BLOCK_END (BLOCKVECTOR_BLOCK (bl, index)) < end)
+	{
+	  if (blocks_printed[index] == 0)
+	    {
+	      if (print_block_frame_labels (BLOCKVECTOR_BLOCK (bl, index), frame, &have_default, stream))
+		values_printed = 1;
+	      blocks_printed[index] = 1;
+	    }
+	  index++;
+	}
+      if (have_default)
+	return 1;
+      if (values_printed && this_level_only)
+	return 1;
+
+      /* After handling the function's top-level block, stop.
+	 Don't continue to its superblock, the block of
+	 per-file symbols.  */
+      if (BLOCK_FUNCTION (block))
+	break;
+      block = BLOCK_SUPERBLOCK (block);
+    }
+
+  if (!values_printed && !this_level_only)
+    {
+      fprintf_filtered (stream, "No catches.\n");
+      fflush (stream);
+    }
+  
+  return values_printed;
+}
+
+static void
+locals_info (args, from_tty)
+     char *args;
+     int from_tty;
+{
+  if (!target_has_stack)
+    error ("No stack.");
+
   print_frame_local_vars (selected_frame, stdout);
 }
 
+static void
+catch_info ()
+{
+  if (!target_has_stack)
+    error ("No stack.");
+
+  print_frame_label_vars (selected_frame, 0, stdout);
+}
+
 static int
 print_frame_arg_vars (frame, stream)
      register FRAME frame;
      register FILE *stream;
 {
-  struct symbol *func;
+  struct symbol *func = get_frame_function (frame);
   register struct block *b;
   int nsyms;
   register int i;
-  register struct symbol *sym;
+  register struct symbol *sym, *sym2;
+  register int values_printed = 0;
 
-  func = get_frame_function (frame);
   if (func == 0)
-    return 0;
+    {
+      fprintf_filtered (stream, "No symbol table info available.\n");
+      fflush (stream);
+      return 0;
+    }
 
   b = SYMBOL_BLOCK_VALUE (func);
   nsyms = BLOCK_NSYMS (b);
@@ -533,23 +789,39 @@ print_frame_arg_vars (frame, stream)
   for (i = 0; i < nsyms; i++)
     {
       sym = BLOCK_SYM (b, i);
-      if (SYMBOL_CLASS (sym) == LOC_ARG || SYMBOL_CLASS (sym) == LOC_REGPARM)
+      if (SYMBOL_CLASS (sym) == LOC_ARG
+	  || SYMBOL_CLASS (sym) == LOC_REF_ARG
+	  || SYMBOL_CLASS (sym) == LOC_REGPARM)
 	{
-	  fprintf (stream, "%s = ", SYMBOL_NAME (sym));
-	  print_variable_value (sym, frame, stream);
-	  fprintf (stream, "\n");
+	  values_printed = 1;
+	  fprint_symbol (stream, SYMBOL_NAME (sym));
+	  fputs_filtered (" = ", stream);
+	  /* We have to look up the symbol because arguments often have
+	     two entries (one a parameter, one a register) and the one
+	     we want is the register, which lookup_symbol will find for
+	     us.  */
+	  sym2 = lookup_symbol (SYMBOL_NAME (sym),
+			b, VAR_NAMESPACE, (int *)NULL, (struct symtab **)NULL);
+	  print_variable_value (sym2, frame, stream);
+	  fprintf_filtered (stream, "\n");
 	  fflush (stream);
 	}
     }
 
+  if (!values_printed)
+    {
+      fprintf_filtered (stream, "No arguments.\n");
+      fflush (stream);
+    }
+
   return 1;
 }
 
 static void
 args_info ()
 {
-  if (selected_frame == 0)
-    error(no_sel_frame);
+  if (!target_has_stack)
+    error ("No stack.");
   print_frame_arg_vars (selected_frame, stdout);
 }
 
@@ -563,6 +835,9 @@ select_frame (frame, level)
 {
   selected_frame = frame;
   selected_frame_level = level;
+  /* Ensure that symbols for this frame are readin.  */
+  if (frame)
+    find_pc_symtab (get_frame_info (frame)->pc);
 }
 
 /* Store the selected frame and its level into *FRAMEP and *LEVELP.  */
@@ -582,7 +857,7 @@ record_selected_frame (frameaddrp, levelp)
 struct block *
 get_selected_block ()
 {
-  if (!have_inferior_p () && !have_core_file_p ())
+  if (!target_has_stack)
     return 0;
 
   if (!selected_frame)
@@ -607,8 +882,6 @@ find_relative_frame (frame, level_offset_ptr)
   register FRAME prev;
   register FRAME frame1, frame2;
 
-  if (frame == 0)
-    error (no_sel_frame);
   /* Going up is simple: just do get_prev_frame enough times
      or until initial frame is reached.  */
   while (*level_offset_ptr > 0)
@@ -661,6 +934,9 @@ frame_command (level_exp, from_tty)
   register FRAME frame, frame1;
   unsigned int level = 0;
 
+  if (!target_has_stack)
+    error ("No stack.");
+
   frame = parse_frame_specification (level_exp);
 
   for (frame1 = get_prev_frame (0);
@@ -683,8 +959,9 @@ frame_command (level_exp, from_tty)
    from the previously selected frame, and print it briefly.  */
 
 static void
-up_command (count_exp)
+up_silently_command (count_exp, from_tty)
      char *count_exp;
+     int from_tty;
 {
   register FRAME frame;
   int count = 1, count1;
@@ -692,11 +969,21 @@ up_command (count_exp)
     count = parse_and_eval_address (count_exp);
   count1 = count;
   
+  if (!target_has_stack)
+    error ("No stack.");
+
   frame = find_relative_frame (selected_frame, &count1);
   if (count1 != 0 && count_exp == 0)
     error ("Initial frame selected; you cannot go up.");
   select_frame (frame, selected_frame_level + count - count1);
+}
 
+static void
+up_command (count_exp, from_tty)
+     char *count_exp;
+     int from_tty;
+{
+  up_silently_command (count_exp, from_tty);
   print_stack_frame (selected_frame, selected_frame_level, 1);
 }
 
@@ -704,8 +991,9 @@ up_command (count_exp)
    from the previously selected frame, and print it briefly.  */
 
 static void
-down_command (count_exp)
+down_silently_command (count_exp, from_tty)
      char *count_exp;
+     int from_tty;
 {
   register FRAME frame;
   int count = -1, count1;
@@ -717,7 +1005,15 @@ down_command (count_exp)
   if (count1 != 0 && count_exp == 0)
     error ("Bottom (i.e., innermost) frame selected; you cannot go down.");
   select_frame (frame, selected_frame_level + count - count1);
+}
 
+
+static void
+down_command (count_exp, from_tty)
+     char *count_exp;
+     int from_tty;
+{
+  down_silently_command (count_exp, from_tty);
   print_stack_frame (selected_frame, selected_frame_level, 1);
 }
 
@@ -727,6 +1023,7 @@ return_command (retval_exp, from_tty)
      int from_tty;
 {
   struct symbol *thisfun = get_frame_function (selected_frame);
+  FRAME_ADDR selected_frame_addr = FRAME_FP (selected_frame);
 
   /* If interactive, require confirmation.  */
 
@@ -742,9 +1039,12 @@ return_command (retval_exp, from_tty)
 	  error ("Not confirmed.");
     }
 
-  /* Do the real work.  Pop until the specified frame is current.  */
+  /* Do the real work.  Pop until the specified frame is current.  We
+     use this method because the selected_frame is not valid after
+     a POP_FRAME.  Note that this will not work if the selected frame
+     shares it's fp with another frame.  */
 
-  while (selected_frame != get_current_frame ())
+  while (selected_frame_addr != FRAME_FP (get_current_frame()))
     POP_FRAME;
 
   /* Then pop that frame.  */
@@ -763,8 +1063,6 @@ return_command (retval_exp, from_tty)
     frame_command ("0", 1);
 }
 
-extern struct cmd_list_element *setlist;
-
 void
 _initialize_stack ()
 {
@@ -781,11 +1079,17 @@ If an argument is given, it is an expression for the value to return.");
   add_com ("up", class_stack, up_command,
 	   "Select and print stack frame that called this one.\n\
 An argument says how many frames up to go.");
+  add_com ("up-silently", class_support, up_silently_command,
+	   "Same as the `up' command, but does not print anything.\n\
+This is useful in command scripts.");
 
   add_com ("down", class_stack, down_command,
 	   "Select and print stack frame called by this one.\n\
 An argument says how many frames down to go.");
   add_com_alias ("do", "down", class_stack, 1);
+  add_com ("down-silently", class_support, down_silently_command,
+	   "Same as the `down' command, but does not print anything.\n\
+This is useful in command scripts.");
 
   add_com ("frame", class_stack, frame_command,
 	   "Select and print a stack frame.\n\
@@ -812,6 +1116,8 @@ With a negative argument, print outermost -COUNT frames.");
 	    "Local variables of current stack frame.");
   add_info ("args", args_info,
 	    "Argument variables of current stack frame.");
+  add_info ("catch", catch_info,
+	    "Exceptions that can be caught in the current stack frame.");
 
 #if 0
   add_cmd ("backtrace-limit", class_stack, set_backtrace_limit_command, 
