@@ -1,31 +1,32 @@
 /* Perform arithmetic and other operations on values, for GDB.
-   Copyright (C) 1986 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1989 Free Software Foundation, Inc.
 
-GDB is distributed in the hope that it will be useful, but WITHOUT ANY
-WARRANTY.  No author or distributor accepts responsibility to anyone
-for the consequences of using it or for whether it serves any
-particular purpose or works at all, unless he says so in writing.
-Refer to the GDB General Public License for full details.
+This file is part of GDB.
 
-Everyone is granted permission to copy, modify and redistribute GDB,
-but only under the conditions described in the GDB General Public
-License.  A copy of this license is supposed to have been given to you
-along with GDB so you can know your rights and responsibilities.  It
-should be in a file named COPYING.  Among other things, the copyright
-notice and this notice must be preserved on all copies.
+GDB is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-In other words, go ahead and share GDB, but don't try to stop
-anyone else from sharing it farther.  Help stamp out software hoarding!
-*/
+GDB is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GDB; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "defs.h"
 #include "param.h"
-#include "symtab.h"
 #include "value.h"
 #include "expression.h"
+#include "target.h"
+#include <string.h>
 
 
 value value_x_binop ();
+value value_subscripted_rvalue ();
 
 value
 value_add (arg1, arg2)
@@ -75,25 +76,34 @@ value_sub (arg1, arg2)
   COERCE_ARRAY (arg1);
   COERCE_ARRAY (arg2);
 
-  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_PTR
-      && 
-      TYPE_CODE (VALUE_TYPE (arg2)) == TYPE_CODE_INT)
+  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_PTR)
     {
-      val = value_from_long (builtin_type_long,
-			     value_as_long (arg1)
-			     - TYPE_LENGTH (TYPE_TARGET_TYPE (VALUE_TYPE (arg1))) * value_as_long (arg2));
-      VALUE_TYPE (val) = VALUE_TYPE (arg1);
-      return val;
-    }
-
-  if (TYPE_CODE (VALUE_TYPE (arg1)) == TYPE_CODE_PTR
-      && 
-      VALUE_TYPE (arg1) == VALUE_TYPE (arg2))
-    {
-      val = value_from_long (builtin_type_long,
-			     (value_as_long (arg1) - value_as_long (arg2))
-			     / TYPE_LENGTH (TYPE_TARGET_TYPE (VALUE_TYPE (arg1))));
-      return val;
+      if (TYPE_CODE (VALUE_TYPE (arg2)) == TYPE_CODE_INT)
+	{
+	  /* pointer - integer.  */
+	  val = value_from_long
+	    (builtin_type_long,
+	     value_as_long (arg1)
+	     - (TYPE_LENGTH (TYPE_TARGET_TYPE (VALUE_TYPE (arg1)))
+		* value_as_long (arg2)));
+	  VALUE_TYPE (val) = VALUE_TYPE (arg1);
+	  return val;
+	}
+      else if (VALUE_TYPE (arg1) == VALUE_TYPE (arg2))
+	{
+	  /* pointer to <type x> - pointer to <type x>.  */
+	  val = value_from_long
+	    (builtin_type_long,
+	     (value_as_long (arg1) - value_as_long (arg2))
+	     / TYPE_LENGTH (TYPE_TARGET_TYPE (VALUE_TYPE (arg1))));
+	  return val;
+	}
+      else
+	{
+	  error ("\
+First argument of `-' is a pointer and second argument is neither\n\
+an integer nor a pointer of the same type.");
+	}
     }
 
   return value_binop (arg1, arg2, BINOP_SUB);
@@ -105,7 +115,59 @@ value
 value_subscript (array, idx)
      value array, idx;
 {
-  return value_ind (value_add (array, idx));
+  if (TYPE_CODE (VALUE_TYPE (array)) == TYPE_CODE_ARRAY
+      && VALUE_LVAL (array) != lval_memory)
+    return value_subscripted_rvalue (array, idx);
+  else
+    return value_ind (value_add (array, idx));
+}
+
+/* Return the value of EXPR[IDX], expr an aggregate rvalue
+   (eg, a vector register) */
+
+value
+value_subscripted_rvalue (array, idx)
+     value array, idx;
+{
+  struct type *elt_type = TYPE_TARGET_TYPE (VALUE_TYPE (array));
+  int elt_size = TYPE_LENGTH (elt_type);
+  int elt_offs = elt_size * value_as_long (idx);
+  value v;
+
+  if (elt_offs >= TYPE_LENGTH (VALUE_TYPE (array)))
+    error ("no such vector element");
+
+  if (TYPE_CODE (elt_type) == TYPE_CODE_FLT) 
+    {
+      if (elt_size == sizeof (float))
+	{
+	  float f = *(float *) (VALUE_CONTENTS (array) + elt_offs);
+	  SWAP_TARGET_AND_HOST (&f, sizeof(f));
+	  v = value_from_double (elt_type, (double) f);
+	}
+      else
+	{
+	  double f = *(double *) (VALUE_CONTENTS (array) + elt_offs);
+	  SWAP_TARGET_AND_HOST (&f, sizeof (f));
+	  v = value_from_double (elt_type, f);
+	}
+    }
+  else
+    {
+      v = allocate_value (elt_type);
+      bcopy (VALUE_CONTENTS (array) + elt_offs,
+	     VALUE_CONTENTS_RAW (v),
+	     TYPE_LENGTH (elt_type));
+    }
+
+  if (VALUE_LVAL (array) == lval_internalvar)
+    VALUE_LVAL (v) = lval_internalvar_component;
+  else
+    VALUE_LVAL (v) = not_lval;
+  VALUE_ADDRESS (v) = VALUE_ADDRESS (array);
+  VALUE_OFFSET (v) = VALUE_OFFSET (array) + elt_offs;
+  VALUE_BITSIZE (v) = elt_size * 8;
+  return v;
 }
 
 /* Check to see if either argument is a structure.  This is called so
@@ -154,12 +216,13 @@ int unop_user_defined_p (op, arg1)
 value
 value_x_binop (arg1, arg2, op, otherop)
      value arg1, arg2;
-     int op, otherop;
+     enum exp_opcode op, otherop;
 {
   value * argvec;
   char *ptr;
   char tstr[13];
-  
+  int static_memfuncp;
+
   COERCE_ENUM (arg1);
   COERCE_ENUM (arg2);
 
@@ -167,7 +230,7 @@ value_x_binop (arg1, arg2, op, otherop)
      arg vector and find the right function to call it with.  */
 
   if (TYPE_CODE (VALUE_TYPE (arg1)) != TYPE_CODE_STRUCT)
-    error ("friend functions not implemented yet");
+    error ("Can't do that binary op on that type");  /* FIXME be explicit */
 
   argvec = (value *) alloca (sizeof (value) * 4);
   argvec[1] = value_addr (arg1);
@@ -175,8 +238,8 @@ value_x_binop (arg1, arg2, op, otherop)
   argvec[3] = 0;
 
   /* make the right function name up */  
-  strcpy(tstr, "operator __");
-  ptr = tstr+9;
+  strcpy(tstr, "operator__");
+  ptr = tstr+8;
   switch (op)
     {
     case BINOP_ADD:	strcpy(ptr,"+"); break;
@@ -219,10 +282,20 @@ value_x_binop (arg1, arg2, op, otherop)
     default:
       error ("Invalid binary operation specified.");
     }
-  argvec[0] = value_struct_elt (arg1, argvec+1, tstr, "structure");
+  argvec[0] = value_struct_elt (&arg1, argvec+1, tstr, &static_memfuncp, "structure");
   if (argvec[0])
-    return call_function (argvec[0], 2, argvec + 1);
-  else error ("member function %s not found", tstr);
+    {
+      if (static_memfuncp)
+	{
+	  argvec[1] = argvec[0];
+	  argvec++;
+	}
+      return target_call_function (argvec[0], 2 - static_memfuncp, argvec + 1);
+    }
+  error ("member function %s not found", tstr);
+#ifdef lint
+  return target_call_function (argvec[0], 2 - static_memfuncp, argvec + 1);
+#endif
 }
 
 /* We know that arg1 is a structure, so try to find a unary user
@@ -234,27 +307,28 @@ value_x_binop (arg1, arg2, op, otherop)
 value
 value_x_unop (arg1, op)
      value arg1;
-     int op;
+     enum exp_opcode op;
 {
   value * argvec;
   char *ptr;
   char tstr[13];
-  
+  int static_memfuncp;
+
   COERCE_ENUM (arg1);
 
   /* now we know that what we have to do is construct our
      arg vector and find the right function to call it with.  */
 
   if (TYPE_CODE (VALUE_TYPE (arg1)) != TYPE_CODE_STRUCT)
-    error ("friend functions not implemented yet");
+    error ("Can't do that unary op on that type");  /* FIXME be explicit */
 
   argvec = (value *) alloca (sizeof (value) * 3);
   argvec[1] = value_addr (arg1);
   argvec[2] = 0;
 
   /* make the right function name up */  
-  strcpy(tstr,"operator __");
-  ptr = tstr+9;
+  strcpy(tstr,"operator__");
+  ptr = tstr+8;
   switch (op)
     {
     case UNOP_PREINCREMENT:	strcpy(ptr,"++"); break;
@@ -267,10 +341,17 @@ value_x_unop (arg1, op)
     default:
       error ("Invalid binary operation specified.");
     }
-  argvec[0] = value_struct_elt (arg1, argvec+1, tstr, "structure");
+  argvec[0] = value_struct_elt (&arg1, argvec+1, tstr, &static_memfuncp, "structure");
   if (argvec[0])
-    return call_function (argvec[0], 1, argvec + 1);
-  else error ("member function %s not found", tstr);
+    {
+      if (static_memfuncp)
+	{
+	  argvec[1] = argvec[0];
+	  argvec++;
+	}
+      return target_call_function (argvec[0], 1 - static_memfuncp, argvec + 1);
+    }
+  error ("member function %s not found", tstr);
 }
 
 /* Perform a binary operation on two integers or two floats.
@@ -326,78 +407,160 @@ value_binop (arg1, arg2, op)
 	}
 
       val = allocate_value (builtin_type_double);
-      *(double *) VALUE_CONTENTS (val) = v;
+      SWAP_TARGET_AND_HOST (&v, sizeof (v));
+      *(double *) VALUE_CONTENTS_RAW (val) = v;
     }
   else
+    /* Integral operations here.  */
     {
-      LONGEST v1, v2, v;
-      v1 = value_as_long (arg1);
-      v2 = value_as_long (arg2);
-
-      switch (op)
+      /* Should we promote to unsigned longest?  */
+      if ((TYPE_UNSIGNED (VALUE_TYPE (arg1))
+	   || TYPE_UNSIGNED (VALUE_TYPE (arg2)))
+	  && (TYPE_LENGTH (VALUE_TYPE (arg1)) >= sizeof (unsigned LONGEST)
+	      || TYPE_LENGTH (VALUE_TYPE (arg1)) >= sizeof (unsigned LONGEST)))
 	{
-	case BINOP_ADD:
-	  v = v1 + v2;
-	  break;
+	  unsigned LONGEST v1, v2, v;
+	  v1 = (unsigned LONGEST) value_as_long (arg1);
+	  v2 = (unsigned LONGEST) value_as_long (arg2);
+	  
+	  switch (op)
+	    {
+	    case BINOP_ADD:
+	      v = v1 + v2;
+	      break;
+	      
+	    case BINOP_SUB:
+	      v = v1 - v2;
+	      break;
+	      
+	    case BINOP_MUL:
+	      v = v1 * v2;
+	      break;
+	      
+	    case BINOP_DIV:
+	      v = v1 / v2;
+	      break;
+	      
+	    case BINOP_REM:
+	      v = v1 % v2;
+	      break;
+	      
+	    case BINOP_LSH:
+	      v = v1 << v2;
+	      break;
+	      
+	    case BINOP_RSH:
+	      v = v1 >> v2;
+	      break;
+	      
+	    case BINOP_LOGAND:
+	      v = v1 & v2;
+	      break;
+	      
+	    case BINOP_LOGIOR:
+	      v = v1 | v2;
+	      break;
+	      
+	    case BINOP_LOGXOR:
+	      v = v1 ^ v2;
+	      break;
+	      
+	    case BINOP_AND:
+	      v = v1 && v2;
+	      break;
+	      
+	    case BINOP_OR:
+	      v = v1 || v2;
+	      break;
+	      
+	    case BINOP_MIN:
+	      v = v1 < v2 ? v1 : v2;
+	      break;
+	      
+	    case BINOP_MAX:
+	      v = v1 > v2 ? v1 : v2;
+	      break;
+	      
+	    default:
+	      error ("Invalid binary operation on numbers.");
+	    }
 
-	case BINOP_SUB:
-	  v = v1 - v2;
-	  break;
-
-	case BINOP_MUL:
-	  v = v1 * v2;
-	  break;
-
-	case BINOP_DIV:
-	  v = v1 / v2;
-	  break;
-
-	case BINOP_REM:
-	  v = v1 % v2;
-	  break;
-
-	case BINOP_LSH:
-	  v = v1 << v2;
-	  break;
-
-	case BINOP_RSH:
-	  v = v1 >> v2;
-	  break;
-
-	case BINOP_LOGAND:
-	  v = v1 & v2;
-	  break;
-
-	case BINOP_LOGIOR:
-	  v = v1 | v2;
-	  break;
-
-	case BINOP_LOGXOR:
-	  v = v1 ^ v2;
-	  break;
-
-	case BINOP_AND:
-	  v = v1 && v2;
-	  break;
-
-	case BINOP_OR:
-	  v = v1 || v2;
-	  break;
-
-	case BINOP_MIN:
-	  v = v1 < v2 ? v1 : v2;
-	  break;
-
-	case BINOP_MAX:
-	  v = v1 > v2 ? v1 : v2;
-	  break;
-
-	default:
-	  error ("Invalid binary operation on numbers.");
+	  val = allocate_value (BUILTIN_TYPE_UNSIGNED_LONGEST);
+	  SWAP_TARGET_AND_HOST (&v, sizeof (v));
+	  *(unsigned LONGEST *) VALUE_CONTENTS_RAW (val) = v;
+	}
+      else
+	{
+	  LONGEST v1, v2, v;
+	  v1 = value_as_long (arg1);
+	  v2 = value_as_long (arg2);
+	  
+	  switch (op)
+	    {
+	    case BINOP_ADD:
+	      v = v1 + v2;
+	      break;
+	      
+	    case BINOP_SUB:
+	      v = v1 - v2;
+	      break;
+	      
+	    case BINOP_MUL:
+	      v = v1 * v2;
+	      break;
+	      
+	    case BINOP_DIV:
+	      v = v1 / v2;
+	      break;
+	      
+	    case BINOP_REM:
+	      v = v1 % v2;
+	      break;
+	      
+	    case BINOP_LSH:
+	      v = v1 << v2;
+	      break;
+	      
+	    case BINOP_RSH:
+	      v = v1 >> v2;
+	      break;
+	      
+	    case BINOP_LOGAND:
+	      v = v1 & v2;
+	      break;
+	      
+	    case BINOP_LOGIOR:
+	      v = v1 | v2;
+	      break;
+	      
+	    case BINOP_LOGXOR:
+	      v = v1 ^ v2;
+	      break;
+	      
+	    case BINOP_AND:
+	      v = v1 && v2;
+	      break;
+	      
+	    case BINOP_OR:
+	      v = v1 || v2;
+	      break;
+	      
+	    case BINOP_MIN:
+	      v = v1 < v2 ? v1 : v2;
+	      break;
+	      
+	    case BINOP_MAX:
+	      v = v1 > v2 ? v1 : v2;
+	      break;
+	      
+	    default:
+	      error ("Invalid binary operation on numbers.");
+	    }
+	  
+	  val = allocate_value (BUILTIN_TYPE_LONGEST);
+	  SWAP_TARGET_AND_HOST (&v, sizeof (v));
+	  *(LONGEST *) VALUE_CONTENTS_RAW (val) = v;
 	}
-
-      val = allocate_value (BUILTIN_TYPE_LONGEST);
-      *(LONGEST *) VALUE_CONTENTS (val) = v;
     }
 
   return val;
@@ -452,7 +615,7 @@ value_equal (arg1, arg2)
     return value_as_double (arg1) == value_as_double (arg2);
   else if ((code1 == TYPE_CODE_PTR && code2 == TYPE_CODE_INT)
 	   || (code2 == TYPE_CODE_PTR && code1 == TYPE_CODE_INT))
-    return value_as_long (arg1) == value_as_long (arg2);
+    return (char *) value_as_long (arg1) == (char *) value_as_long (arg2);
   else if (code1 == code2
 	   && ((len = TYPE_LENGTH (VALUE_TYPE (arg1)))
 	       == TYPE_LENGTH (VALUE_TYPE (arg2))))
@@ -487,13 +650,22 @@ value_less (arg1, arg2)
   code2 = TYPE_CODE (VALUE_TYPE (arg2));
 
   if (code1 == TYPE_CODE_INT && code2 == TYPE_CODE_INT)
-    return value_as_long (arg1) < value_as_long (arg2);
+    {
+      if (TYPE_UNSIGNED (VALUE_TYPE (arg1))
+       || TYPE_UNSIGNED (VALUE_TYPE (arg2)))
+	return (unsigned)value_as_long (arg1) < (unsigned)value_as_long (arg2);
+      else
+	return value_as_long (arg1) < value_as_long (arg2);
+    }
   else if ((code1 == TYPE_CODE_FLT || code1 == TYPE_CODE_INT)
 	   && (code2 == TYPE_CODE_FLT || code2 == TYPE_CODE_INT))
     return value_as_double (arg1) < value_as_double (arg2);
   else if ((code1 == TYPE_CODE_PTR || code1 == TYPE_CODE_INT)
 	   && (code2 == TYPE_CODE_PTR || code2 == TYPE_CODE_INT))
-    return value_as_long (arg1) < value_as_long (arg2);
+    {
+      /* FIXME, this assumes that host and target char *'s are the same! */
+      return (char *) value_as_long (arg1) < (char *) value_as_long (arg2);
+    }
   else
     error ("Invalid type combination in ordering comparison.");
 }
